<?php

namespace App\Http\Controllers\Admin\AdminAuth;

use App\Http\Controllers\Controller;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Illuminate\Support\Facades\Auth;
use Hesto\MultiAuth\Traits\LogsoutGuard;

use Illuminate\Http\Request;

use App\Models\Admin\Admin;

use Hash;

class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers, LogsoutGuard {
        LogsoutGuard::logout insteadof AuthenticatesUsers;
    }

    /**
     * Where to redirect users after login / registration.
     *
     * @var string
     */
    public $redirectTo = '/admin/home';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('admin.guest', ['except' => 'logout']);
    }

    /**
     * Show the application's login form.
     *
     * @return \Illuminate\Http\Response
     */
    public function showLoginForm()
    {
        return view('admin.auth.login');
    }

    public function login(Request $request)
    {
        $request->validate([
            'email' => 'required',
            'password' => 'required',
            // 'one_time_password' => 'required'
        ]);

        $admin = Admin::where('email', $request->email)->first();
        $validCredentials = Hash::check($request->password, $admin->getAuthPassword());

        if ($validCredentials) {
            if($admin->google2fa_secret == '' || $admin->google2fa_secret == null){
                return back()->with('error', "An unknown error has occurred. Please try again.")->withInput();
            }
            $google2fa = app('pragmarx.google2fa');
            $valid = $google2fa->verifyKey($admin->google2fa_secret, $request->one_time_password);
            if (!$valid) {
                return back()->with('error', "The 'One Time Password' typed was wrong")->withInput();
            }

            $credentials = $request->only('email', 'password');
            if (Auth::guard('admin')->attempt($credentials)) {
                return redirect()->route('admin.home');
            }
        }

        return redirect()->route('admin.login')->with('error', 'These credentials do not match our records.')->withInput();
    }

    /**
     * Get the guard to be used during authentication.
     *
     * @return \Illuminate\Contracts\Auth\StatefulGuard
     */
    protected function guard()
    {
        return Auth::guard('admin');
    }

    public function logout() {
        Auth::guard('admin')->logout();
        return redirect()->route('admin.login')->with('success','Admin logout successfully.');
    }

    public function logoutToPath() {
        return '/admin/login';
    }
}
