<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Mail\GoogleAuthTokenMail;

use App\Models\Admin\Admin;
use App\Models\Admin\Permission;
use App\Models\Admin\Role;

use BaconQrCode\Renderer\ImageRenderer;
use BaconQrCode\Renderer\Image\ImagickImageBackEnd;
use BaconQrCode\Renderer\RendererStyle\RendererStyle;
use BaconQrCode\Writer;

use DataTables;
use Response;
use Mail;
use Redirect;
use Auth;
use Hash;
use Image;
use QrCode;
use Helper;
use Str;

use App\Traits\Authorizable;

class AdminController extends Controller
{
    use Authorizable;

    public function __construct() {
        $this->middleware('auth:admin');
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            // $raw = Admin::query();
            $raw = Admin::with('roles')->get();
            // $raw = $raw->reject(function ($raw, $key) {
            //     return $raw->hasRole('admin');
            // });
            $datatable = app('datatables')->of($raw)
            ->editColumn('created_at', function ($raw) {
                if($raw->created_at == null || $raw->created_at == ''){
                    return '-';
                }else{
                    // return date('Y/m/d', strtotime($raw->created_at->toFormattedDateString()));
                    // return $raw->created_at->toFormattedDateString();
                    return date('Y-m-d H:i:s A', strtotime($raw->created_at));
                }
            })
            // ->editColumn('google2fa_secret', function ($raw) {
            //     if($raw->google2fa_secret == null || $raw->google2fa_secret == ''){
            //         return '-';
            //     }else{
            //         $google2fa = app('pragmarx.google2fa');
            //         $QR_Image = $google2fa->getQRCodeUrl(
            //             config('app.name'),
            //             $raw->email,
            //             $raw->google2fa_secret
            //         );
            //     return QrCode::size(100)->generate($QR_Image);
            //         // $google2fa_url = custom_generate_qrcode_url($QR_Image);
            //         // return '<img src="{{ $google2fa_url }}" alt="">';
            //         // return $QR_Image;
            //     }
            // })
            ->editColumn('qrcode_base64', function ($raw) {
                if($raw->qrcode_base64 == null || $raw->qrcode_base64 == ''){
                    return '-';
                }else{
                    return '<img src="data:image/png;base64, '.$raw->qrcode_base64.' "  width="100"/>';
                }
            })
            ->addColumn('role', function ($raw) {
                // return removeDash($raw->roles->first()->name);
                return Helper::ArrayToString($raw->getRoleNames()->toArray());
            })
            ->addColumn('action', function ($raw) {
                $html = '';
                // echo '<pre>';print_r($raw->roles);exit;
                // if($raw->roles->first()->name == 'Admin' || $raw->roles->first()->name == 'admin'){
                // }else{
                    if(auth()->user()->can('edit_admins')){
                        $html .= '<a href="'.route("admin.admins.edit", $raw->id).'" class="label theme-bg2 text-white f-12">Edit</a>';
                    }
                    if(!(Helper::checkRole($raw->roles->first()->name))){
                        if(auth()->user()->can('delete_admins')){
                            $html .= '<a href="javascript:void(0)" class="label theme-bg text-white f-12 delete-admin-btn" data-url="'. route("admin.admins.destroy", $raw->id) .'">Delete</a>';
                        }
                    }
                    // if(auth()->user()->can('delete_admins')){
                    //     $html .= '<a href="javascript:void(0)" class="label theme-bg2 text-white f-12 send_key_btn" data-id="'.$raw->id.'" data-url="'. route("admin.admins.send_key") .'">Send Key</a>';
                    // }
                // }
                return $html;
            });
            return $datatable->make(true);
        }
        return view('admin.admin.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $roles = Role::where('guard_name', 'admin')->pluck('name', 'id');
        return view('admin.admin.create', compact('roles'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // echo 'strrrr';exit;
        $this->validate($request, [
            'name' => 'bail|required|min:2',
            'email' => 'required|email|unique:admins',
            'password' => 'required|min:6',
            'roles' => 'required|min:1'
        ]);

        // $google2fa = app('pragmarx.google2fa');
        // $SecretKey =  $google2fa->generateSecretKey();

        // echo '<pre>';print_r($request->all());exit;

        // hash password
        $request->merge(['password' => bcrypt($request->get('password'))]);

        // Create the admin
        if ( $admin = Admin::create($request->except('roles', 'permissions')) ) {

            // Mail::to($admin->email)->send(new GoogleAuthTokenMail($admin));

            Helper::assignRoleAndPermissions($request, $admin);

            $google2fa = app('pragmarx.google2fa');
            $prefix = Str::padRight($admin->id, 10, 'X');
            $secret = $google2fa->generateSecretKey(16, $prefix);
            $admin->google2fa_secret = $secret;

            $g2faUrl = $google2fa->getQRCodeUrl(config('app.name'), $admin->email, $secret);
            $writer = new Writer(
                new ImageRenderer(
                    new RendererStyle(400),
                    new ImagickImageBackEnd()
                )
            );
            $qr_code = base64_encode($writer->writeString($g2faUrl));

            $admin->qrcode_base64 = $qr_code;
            $admin->save();

            return redirect()->route('admin.admins.index')->with('success','Admin has been created.');

        } else {
            return redirect()->route('admin.admins.index')->with('error','Unable to create admin.');
        }

        return redirect()->route('admin.admins.index')->with('error','Unable to create admin.');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $admin = Admin::find($id);
        $roles = Role::where('guard_name', 'admin')->pluck('name', 'id');
        // $permissions = Permission::all('name', 'id');
        $permissions = Permission::where('guard_name', 'admin')->get('name', 'id');

        return view('admin.admin.edit', compact('admin', 'roles', 'permissions'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Admin $admin)
    {
        $this->validate($request, [
            'name' => 'bail|required|min:2',
            'email' => 'required|email|unique:admins,email,' . $admin->id,
            'roles' => 'required|min:1'
        ]);

        // $SecretKey = '';
        // if($admin->google2fa_secret == '' || $admin->google2fa_secret == null){
        //     $google2fa = app('pragmarx.google2fa');
        //     $SecretKey = $google2fa->generateSecretKey();
        //     // Mail::to($admin->email)->send(new GoogleAuthTokenMail($admin));
        // }
        $input = $request->except(['_token', 'roles', 'permissions', 'password']);
        // $input['google2fa_secret'] = $SecretKey;

        // Update admin
        $admin->fill($input);

        // echo '<pre>';
        // print_r($admin->toArray());
        // exit;

        // check for password change
        if($request->get('password')) {
            $admin->password = bcrypt($request->get('password'));
        }

        // Handle the admin roles
        Helper::assignRoleAndPermissions($request, $admin);

        if($admin->qrcode_base64 == '' || $admin->google2fa_secret == ''){
            $google2fa = app('pragmarx.google2fa');
            $prefix = Str::padRight($admin->id, 10, 'X');
            $secret = $google2fa->generateSecretKey(16, $prefix);
            $admin->google2fa_secret = $secret;

            $g2faUrl = $google2fa->getQRCodeUrl(config('app.name'), $admin->email, $secret);
            $writer = new Writer(
                new ImageRenderer(
                    new RendererStyle(400),
                    new ImagickImageBackEnd()
                )
            );
            $qr_code = base64_encode($writer->writeString($g2faUrl));

            $admin->qrcode_base64 = $qr_code;
        }
        $admin->save();

        return redirect()->route('admin.admins.index')->with('success','Admin has been updated.');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, $id)
    {
        $admin = Admin::find($id);
        if((Helper::checkRole($admin->roles->first()->name))){
            return response()->json(['status' => 'error', 'msg' => "Not delete admin."]);
        }
        //if not found
        if(empty($admin)){
            if ($request->ajax()) {
                return response()->json(['status' => 'error', 'msg' => "Some problem delete admin."]);
            }
            return Redirect::route('admin.admins.index')->with('error','Some problem delete admin.');
        }

        //delete
        $admin->delete();
        if ($request->ajax()) {
            return response()->json(['status' => 'success', 'msg' => "Admin Deleted successfully."]);
        }
        return Redirect::route('admin.admins.index')->with('success','Admin delete successfully.');
    }

    public function BulkRemove(Request $request)
    {
        $ids = $request->id;
        // echo '<pre>';print_r($ids);exit;
        $admins = Admin::whereIn('id',$ids)->get();
        if($admins->isEmpty()){
            return response()->json(['status' => 'error', 'msg' => "Some problem delete admin."]);
        }
        Admin::whereIn('id',$ids)->delete();
        return response()->json(['status' => 'success', 'msg' => "Admin deleted successfully."]);
    }

    public function SendKey(Request $request)
    {
        $admin = Admin::find($request->id);

        //if not found
        if(empty($admin)){
            if ($request->ajax()) {
                return response()->json(['status' => 'error', 'msg' => "Some problem send key admin."]);
            }
            return Redirect::route('admin.admins.index')->with('error','Some problem send key admin.');
        }

        //send key
        Mail::to($admin->email)->send(new GoogleAuthTokenMail($admin));

        if ($request->ajax()) {
            return response()->json(['status' => 'success', 'msg' => "Send key successfully."]);
        }
        return Redirect::route('admin.admins.index')->with('success','Send key successfully.');
    }


}
