<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Admin\Category;
use App\Models\Admin\Attribute;

use DataTables;
use Response;
use Mail;
use Redirect;
use Auth;
use Hash;
use Image;
use Helper;
use Illuminate\Validation\Rule;

class AttributeController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $raw = Attribute::with('category');

            //for category filter
            $category_id = $request->category_id;
            if ($category_id > 0) {
                $raw = $raw->where('category_id', $category_id);
            }

            //for status filter
            if ($request->has('status')) {
                $raw = $raw->where('is_active', '=', $request->status);
            }

            $raw->get();

            $datatable = app('datatables')->of($raw)

            ->editColumn('category_id', function ($raw) {
                if($raw->category_id == null || $raw->category_id == ''){
                    return '-';
                }else{
                    return $raw->category->name;
                }
            })

            ->editColumn('created_at', function ($raw) {
                if($raw->created_at == null || $raw->created_at == ''){
                    return '-';
                }else{
                    return date('Y-m-d H:i:s A', strtotime($raw->created_at));
                }
            })
            ->editColumn('is_active', function ($raw) {
                if(in_array('admin', Auth::user()->roles->pluck('name')->toArray())){
                    if($raw->is_active == 1){
                        return '<span class="badge badge-success attribute_status active" data-url="'.route('admin.attributes.status', $raw->id).'">Active</span>';
                    }else{
                        return '<span class="badge badge-dark attribute_status inactive" data-url="'.route('admin.attributes.status', $raw->id).'">Deactive</span>';
                    }
                }
                else{
                    if($raw->status == 1){
                        return '<span class="badge badge-success">Active</span>';
                    }else{
                        return '<span class="badge badge-dark">Deactive</span>';
                    }
                }
            })
            ->addColumn('action', function ($raw) {
                $html = '';
                if(auth()->user()->can('edit_categories')){
                    $html .= '<a href="'.route("admin.attributes.edit", $raw->id).'" class="label theme-bg2 text-white f-12">Edit</a>';
                }
                if(auth()->user()->can('delete_categories')){
                    $html .= '<a href="javascript:void(0)" class="label theme-bg text-white f-12 delete-attribute-btn" data-url="'. route("admin.attributes.destroy", $raw->id) .'">Delete</a>';
                }
                return $html;
            });
            return $datatable->make(true);
        }
        $categories = Category::Active()->where('parent_id','<>', null)->orderby('name', 'asc')->get();
        return view('admin.attribute.index', compact('categories'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $categories = Category::Active()->where('parent_id','<>', null)->orderby('name', 'asc')->get();
        return view('admin.attribute.create', compact('categories'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'category_id' => 'required',
            'code' => 'required',
            'name' => [
                'required',
                Rule::unique('attributes')->where(function ($query) use($request) {
                    return $query->where('name', $request->name)
                    ->where('category_id', $request->category_id);
                }),
            ],
        ],[
            'name.unique' => 'Given attribute already exist in this category',
        ]);

        $input = $request->only('category_id', 'name', 'code');

        if(Attribute::create($input)) {
            return redirect()->route('admin.attributes.index')->with('success','Attribute create successfully.');
        }

        return redirect()->back()->with('error','Unable to create attribute.');
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Attribute $attribute)
    {
        $categories = Category::Active()->where('parent_id','<>', null)->orderby('name', 'asc')->get();
        return view('admin.attribute.edit', compact('categories', 'attribute'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Attribute $attribute)
    {
        $this->validate($request, [
            'name' => [
                'required',
                Rule::unique('attributes')->where(function ($query) use($request, $attribute) {
                    return $query->where('name', $request->name)
                    ->where('category_id', $request->category_id)
                    ->whereNotIn('id', [$attribute->id]);
                }),
            ],
        ],[
            'name.unique' => 'Given attribute already exist in this category',
        ]);

        $input = $request->only('category_id', 'name', 'code');

        if($attribute->update($input) ) {
            return redirect()->route('admin.attributes.index')->with('success','Attribute update successfully.');
        }

        return redirect()->back()->with('error','Unable to update attribute.');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Request $request, Attribute $attribute)
    {
        //if not found
        if(empty($attribute)){
            if ($request->ajax()) {
                return response()->json(['status' => 'error', 'msg' => "Some problem delete attribute."]);
            }
            return Redirect::route('admin.attributes.index')->with('error','Some problem delete attribute.');
        }

        //delete
        $attribute->delete();
        if ($request->ajax()) {
            return response()->json(['status' => 'success', 'msg' => "Attribute Deleted successfully."]);
        }
        return Redirect::route('admin.attributes.index')->with('success','Attribute delete successfully.');
    }

    public function BulkRemove(Request $request)
    {
        $ids = $request->id;
        $attributes = Attribute::whereIn('id',$ids)->get();
        if($attributes->isEmpty()){
            return response()->json(['status' => 'error', 'msg' => "Some problem delete attributes."]);
        }
        Attribute::whereIn('id',$ids)->delete();
        return response()->json(['status' => 'success', 'msg' => "Attributes deleted successfully."]);
    }

     public function Status(Request $request){
        $attribute = Attribute::find($request->id);
        $attribute->is_active = $request->status;
        $attribute->save();
        return response()->json(['status' => 'success']);
    }
}
