<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Admin\Attribute;
use App\Models\Admin\AttributeValue;

use DataTables;
use Response;
use Mail;
use Redirect;
use Auth;
use Hash;
use Image;
use Helper;
use Illuminate\Validation\Rule;

class AttributeValueController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $raw = AttributeValue::with('attribute');

            //for category filter
            $attribute_id = $request->attribute_id;
            if ($attribute_id > 0) {
                $raw = $raw->where('attribute_id', $attribute_id);
            }

            //for status filter
            if ($request->has('status')) {
                $raw = $raw->where('is_active', '=', $request->status);
            }

            $raw->get();

            $datatable = app('datatables')->of($raw)

            ->editColumn('attribute_id', function ($raw) {
                if($raw->attribute_id == null || $raw->attribute_id == ''){
                    return '-';
                }else{
                    return $raw->attribute->name;
                }
            })

            ->editColumn('created_at', function ($raw) {
                if($raw->created_at == null || $raw->created_at == ''){
                    return '-';
                }else{
                    return date('Y-m-d H:i:s A', strtotime($raw->created_at));
                }
            })
            ->editColumn('is_active', function ($raw) {
                if(in_array('admin', Auth::user()->roles->pluck('name')->toArray())){
                    if($raw->is_active == 1){
                        return '<span class="badge badge-success attribute_value_status active" data-url="'.route('admin.attribute-values.status', $raw->id).'">Active</span>';
                    }else{
                        return '<span class="badge badge-dark attribute_value_status inactive" data-url="'.route('admin.attribute-values.status', $raw->id).'">Deactive</span>';
                    }
                }
                else{
                    if($raw->status == 1){
                        return '<span class="badge badge-success">Active</span>';
                    }else{
                        return '<span class="badge badge-dark">Deactive</span>';
                    }
                }
            })
            ->addColumn('action', function ($raw) {
                $html = '';
                if(auth()->user()->can('edit_categories')){
                    $html .= '<a href="'.route("admin.attribute-values.edit", $raw->id).'" class="label theme-bg2 text-white f-12">Edit</a>';
                }
                if(auth()->user()->can('delete_categories')){
                    $html .= '<a href="javascript:void(0)" class="label theme-bg text-white f-12 delete-attributes_value-btn" data-url="'. route("admin.attribute-values.destroy", $raw->id) .'">Delete</a>';
                }
                return $html;
            });
            return $datatable->make(true);
        }
        $attributes = Attribute::Active()->get();
        return view('admin.attribute_value.index', compact('attributes'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $attributes = Attribute::Active()->get();
        return view('admin.attribute_value.create', compact('attributes'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'attribute_id' => 'required',
            'value' => [
                'required',
                Rule::unique('attribute_values')->where(function ($query) use($request) {
                    return $query->where('value', $request->value)
                    ->where('attribute_id', $request->attribute_id);
                }),
            ],
        ],[
            'value.unique' => 'Given value already exist in this attribute',
        ]);

        $input = $request->only('attribute_id', 'value');

        if(AttributeValue::create($input)) {
            return redirect()->route('admin.attribute-values.index')->with('success','Attribute Value create successfully.');
        }

        return redirect()->back()->with('error','Unable to create attribute value.');
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(AttributeValue $attribute_value)
    {
        $attributes = Attribute::Active()->get();
        return view('admin.attribute_value.edit', compact('attribute_value', 'attributes'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, AttributeValue $attribute_value)
    {
        $this->validate($request, [
            'attribute_id' => 'required',
            'value' => [
                'required',
                Rule::unique('attribute_values')->where(function ($query) use($request, $attribute_value) {
                    return $query->where('value', $request->value)
                    ->where('attribute_id', $request->attribute_id)
                    ->whereNotIn('id', [$attribute_value->id]);
                }),
            ],
        ],[
            'value.unique' => 'Given value already exist in this attribute',
        ]);

        $input = $request->only('attribute_id', 'value');

        if($attribute_value->update($input) ) {
            return redirect()->route('admin.attribute-values.index')->with('success','Attribute Value update successfully.');
        }

        return redirect()->back()->with('error','Unable to update attribute value.');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Request $request, AttributeValue $attribute_value)
    {
        //if not found
        if(empty($attribute_value)){
            if ($request->ajax()) {
                return response()->json(['status' => 'error', 'msg' => "Some problem delete attribute value."]);
            }
            return Redirect::route('admin.attributes.index')->with('error','Some problem delete attribute value.');
        }

        //delete
        $attribute_value->delete();
        if ($request->ajax()) {
            return response()->json(['status' => 'success', 'msg' => "Attribute value Deleted successfully."]);
        }
        return Redirect::route('admin.attributes.index')->with('success','Attribute value delete successfully.');
    }

    public function BulkRemove(Request $request)
    {
        $ids = $request->id;
        $attribute_values = AttributeValue::whereIn('id',$ids)->get();
        if($attribute_values->isEmpty()){
            return response()->json(['status' => 'error', 'msg' => "Some problem delete attribute values."]);
        }
        AttributeValue::whereIn('id',$ids)->delete();
        return response()->json(['status' => 'success', 'msg' => "Attribute values deleted successfully."]);
    }

    public function Status(Request $request){
        $attribute = Attribute::find($request->id);
        $attribute->is_active = $request->status;
        $attribute->save();
        return response()->json(['status' => 'success']);
    }
}
