<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Admin\Banner;

use DataTables;
use Response;
use Mail;
use Excel;
use Redirect;
use Image;
use File;
use Storage;
use DB;
use Helper;

use \Cviebrock\EloquentSluggable\Services\SlugService;

use App\Traits\Authorizable;

class BannerController extends Controller
{
    use Authorizable;
    
    public function __construct() {
        $this->middleware('auth:admin');
        //$this->middleware('admin');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $raw = Banner::query();
            $datatable = app('datatables')->of($raw)
            ->editColumn('created_at', function ($raw) {
                if($raw->created_at == ''){
                    return '-';
                }else{
                    return date('Y-m-d H:i:s A', strtotime($raw->created_at));
                }
            })
    
            ->editColumn('image', function ($raw) {
                if($raw->image == '' || $raw->image == null){
                    return '-';
                }else{
                    return '<img src="'.asset('storage/banners/thumbnail/'.$raw->image).'" width="100">';
                }
            })
    
            ->editColumn('status', function ($raw) {
                if($raw->is_active == 1){
                    return '<span class="badge badge-success banner_status active" data-url="'.route('admin.banners.status', $raw->id).'">Active</span>';
                }else{
                    return '<span class="badge badge-dark banner_status inactive" data-url="'.route('admin.banners.status', $raw->id).'">Deactive</span>';
                }
            })
    
            ->addColumn('action', function ($raw) {
                $html = '';
                if(auth()->user()->can('edit_banners')){
                    $html .= '<a href="'.route("admin.banners.edit", $raw->id).'" class="label theme-bg2 text-white f-12">Edit</a>  ';
                }
                if(auth()->user()->can('delete_banners')){
                    $html .= '<a class="label theme-bg text-white f-12 delete-banner-btn" href="javascript:void(0)" data-url="'. route("admin.banners.destroy", $raw->id) .'">Delete</a>';
                }
                return $html;
            });
            
            return $datatable->make(true);
        }
        return view('admin.banner.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.banner.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // dd($request->all());
        $this->validate($request, [
            'caption' => 'required|string|unique:banners',
    		'image' => 'image|mimes:jpeg,png,jpg|max:50480',
        ]);

        $input = $request->except(['_token']);

        if ($request->hasFile('image')) {
            $file = $request->file('image');
            $image_name = SlugService::createSlug(Banner::class, 'slug', $request->caption);
            $file_name = $image_name . '.webp';

            //originals image save
            $original_file_path = '/';
            Helper::createDirecrotoryIfNotExist('banners', $original_file_path);
            Storage::disk('banners')->putFileAs($original_file_path, $file, $file_name);

            //thumbnail image save
            $thumbnailPath = '/thumbnail/';
            Helper::createDirecrotoryIfNotExist('banners', $thumbnailPath);
            $thumb_img = Image::make($file->getRealPath())->fit(100, 100, function ($constraint) {
                $constraint->aspectRatio(); //maintain image ratio
            });
            $thumb_img->stream();
            Storage::disk('banners')->put($thumbnailPath.$file_name, $thumb_img);

            $input['image'] = $file_name;
        }

        return DB::transaction(function() use($input){
            Banner::Create($input);
            return Redirect::route('admin.banners.index')->with('success','Banner create successfully.');
        });
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $banner = Banner::find($id);
        if(empty($banner)){
            return Redirect::route('admin.banners.index')->with('error','Banner not found.');
        }
        return view('admin.banner.edit',compact('banner'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Banner $banner)
    {
        $this->validate($request, [
            'caption' => 'required|unique:banners,caption,' . $banner->id,
        ]);

        $input = $request->except(['_token','_method']);

        if ($request->hasFile('image')) {
            $file = $request->file('image');
            $image_name = SlugService::createSlug(Banner::class, 'slug', $request->caption);
            $file_name = $image_name . '.webp';

            //originals image save
            $original_file_path = '/';
            Helper::createDirecrotoryIfNotExist('banners', $original_file_path);
            Helper::deleteFileIfExist('banners', $original_file_path . $banner->image);
            Storage::disk('banners')->putFileAs($original_file_path, $file, $file_name);

            //thumbnail image save
            $thumbnail_path = '/thumbnail/';
            Helper::createDirecrotoryIfNotExist('banners', $thumbnail_path);
            Helper::deleteFileIfExist('banners', $thumbnail_path . $banner->image);
            $img_thumbnail = Image::make($file->getRealPath())->fit(50, 50, function ($constraint) {
                $constraint->aspectRatio(); //maintain image ratio
            });
            $img_thumbnail->stream();
            Storage::disk('banners')->put($thumbnail_path.$file_name, $img_thumbnail);

            $input['image'] = $file_name;
        }

        return DB::transaction(function() use($input, $banner){
            $banner->update($input);
            return Redirect::route('admin.banners.index')->with('success','Banner update successfully.');
        });
        
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, $id)
    {
        $banner = Banner::find($id);

        //delete old file
        $originalsImgPath = '/';
        $thumbnailPath = '/thumbnail/';
        if(Helper::checkFileExist('banners', $originalsImgPath)) {
            Helper::deleteFileIfExist('banners', $originalsImgPath . $banner->image);
        }
        if(Helper::checkFileExist('banners', $thumbnailPath)) {
            Helper::deleteFileIfExist('banners', $thumbnailPath . $banner->image);
        }
        $banner->delete();
        if ($request->ajax()) {
            return response()->json(['status' => 'success', 'msg' => "Banner deleted successfully."]);
        }
        return Redirect::route('admin.banners.index')->with('success','Banner deleted successfully.');
        
    }

    public function BulkRemove(Request $request)
    {
        $ids = $request->id;
        $banners = Banner::whereIn('id', $ids)->get();
        foreach($banners as $banner){
            $originalsImgPath = '/';
            $thumbnailPath = '/thumbnail/';
            if(Helper::checkFileExist('banners', $originalsImgPath)) {
                Helper::deleteFileIfExist('banners', $originalsImgPath . $banner->image);
            }
            if(Helper::checkFileExist('banners', $thumbnailPath)) {
                Helper::deleteFileIfExist('banners', $thumbnailPath . $banner->image);
            }
            $banner->delete();
        }
        // Category::whereIn('id',$ids)->delete();
        return response()->json(['status' => 'success', 'msg' => "Banner deleted successfully."]);
        
    }

    public function Status(Request $request){
        $banner = Banner::find($request->id);
        $banner->is_active = (int)$request->status;
        $banner->save();
        return response()->json(['status' => 'success']);
    }

}
