<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Admin\Blog;
use App\Models\Admin\BlogCategory;

use DataTables;
use Response;
use Mail;
use Excel;
use Redirect;
use Image;
use File;
use Storage;
use DB;

use App\Traits\Authorizable;

class BlogCategoryController extends Controller
{
    use Authorizable;
    
    public function __construct() {
        $this->middleware('auth:admin');
        //$this->middleware('admin');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $raw = BlogCategory::query();
            $datatable = app('datatables')->of($raw)
    
            ->editColumn('is_active', function ($raw) {
                if($raw->is_active == 1){
                    return '<span class="badge badge-success blog_category_status active" data-url="'.route('admin.blog-categories.status', $raw->id).'">Active</span>';
                }else{
                    return '<span class="badge badge-dark blog_category_status inactive" data-url="'.route('admin.blog-categories.status', $raw->id).'">Deactive</span>';
                }
            })
    
            ->editColumn('created_at', function ($raw) {
                if($raw->created_at == ''){
                    return '-';
                }else{
                    return date('Y-m-d H:i:s A', strtotime($raw->created_at));
                }
            })
    
            ->addColumn('action', function ($raw) {
                $html = '';
                if(auth()->user()->can('edit_blog-categories')){
                    $html .= '<a href="'.route("admin.blog-categories.edit", $raw->id).'" class="label theme-bg2 text-white f-12">Edit</a>  ';
                }
                if(auth()->user()->can('delete_blog-categories')){
                    $html .= '<a class="label theme-bg text-white f-12 delete-blog-category-btn" href="javascript:void(0)" data-url="'. route("admin.blog-categories.destroy", $raw->id) .'">Delete</a>';
                }
                return $html;
            });
            
            return $datatable->make(true);
        }
        return view('admin.blog_categories.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.blog_categories.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // dd($request->all());
        $this->validate($request, [
            'name' => 'required|string|unique:blog_categories',    		
        ]);

        $input = $request->except(['_token']);        
        return DB::transaction(function() use($input){
            BlogCategory::Create($input);
            return Redirect::route('admin.blog-categories.index')->with('success','Blog Category create successfully.');
        });
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $blog_category = BlogCategory::find($id);
        if(empty($blog_category)){
            return Redirect::route('admin.blog-categories.index')->with('error','Category not found.');
        }
        return view('admin.blog_categories.edit',compact('blog_category'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, BlogCategory $blog_category)
    {
        $this->validate($request, [
            'name' => 'required|unique:blog_categories,name,'.$blog_category->id,            
        ]);

        $input = $request->except(['_token','_method']);
        return DB::transaction(function() use($input, $blog_category){
            $blog_category->update($input);
            return Redirect::route('admin.blog-categories.index')->with('success','Blog Category update successfully.');
        });
        
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, $id)
    {
        $blog_category = BlogCategory::find($id);

        $blog_count = $blog_category->blogs->count();
        if($blog_count > 0){
            if ($request->ajax()) {
                return response()->json(['status' => 'error', 'msg' => "This Blog Category already in used. so you have only edit this category."]);    
            }
            return Redirect::route('admin.blog-categories.index')->with('error', 'This Blog Category already in used. so you have only edit this category.');

        }else{
           
            $blog_category->delete();
            if ($request->ajax()) {
                return response()->json(['status' => 'success', 'msg' => "Blog Category deleted successfully."]);
            }
            return Redirect::route('admin.blog-categories.index')->with('success','Blog Category deleted successfully.');
            
        }
        
    }

    public function BulkRemove(Request $request)
    {
        $ids = $request->id;
        $blog_count = Blog::whereIn('blog_category_id', $ids)->count();
        if($blog_count > 0){
            return response()->json(['status' => 'error', 'msg' => "This Blog Category already in used. so you have only edit this category."]);
        }else{
            BlogCategory::whereIn('id',$ids)->delete();
            return response()->json(['status' => 'success', 'msg' => "Blog Category deleted successfully."]);
        }
        
    }

    public function Status(Request $request){
        $category = BlogCategory::find($request->id);
        $category->is_active = (int) $request->status;
        $category->save();
        return response()->json(['status' => 'success']);
    }
}
