<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Admin\Blog;
use App\Models\Admin\BlogCategory;

use DataTables;
use Response;
use Mail;
use Excel;
use Redirect;
use Image;
use File;
use Storage;
use DB;
use Helper;

use \Cviebrock\EloquentSluggable\Services\SlugService;

use App\Traits\Authorizable;

class BlogController extends Controller
{
    use Authorizable;
    
    public function __construct() {
        $this->middleware('auth:admin');
        //$this->middleware('admin');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $raw = Blog::query();
            $datatable = app('datatables')->of($raw)
            ->editColumn('created_at', function ($raw) {
                if($raw->created_at == ''){
                    return '-';
                }else{
                    return date('Y-m-d H:i:s A', strtotime($raw->created_at));
                }
            })
    
            ->editColumn('image', function ($raw) {
                if($raw->image == '' || $raw->image == null){
                    return '-';
                }else{
                    return '<img src="'.asset('storage/blogs/thumbnail/'.$raw->image).'" width="100">';
                }
            })
    
            ->editColumn('blog_category_id', function ($raw) {
                
                if($raw->blog_category_id == '' || $raw->blog_category_id == null){
                    return '-';
                }else{
                    return $raw->blog_category->name;
                }
            })
    
            ->editColumn('is_active', function ($raw) {
                if($raw->is_active == 1){
                    return '<span class="badge badge-success blog_status active" data-url="'.route('admin.blogs.status', $raw->id).'">Active</span>';
                }else{
                    return '<span class="badge badge-dark blog_status inactive" data-url="'.route('admin.blogs.status', $raw->id).'">Deactive</span>';
                }
            })
    
            ->addColumn('action', function ($raw) {
                $html = '';
                if(auth()->user()->can('edit_blogs')){
                    $html .= '<a href="'.route("admin.blogs.edit", $raw->id).'" class="label theme-bg2 text-white f-12">Edit</a>  ';
                }
                if(auth()->user()->can('delete_blogs')){
                    $html .= '<a class="label theme-bg text-white f-12 delete-blog-btn" href="javascript:void(0)" data-url="'. route("admin.blogs.destroy", $raw->id) .'">Delete</a>';
                }
                return $html;
            });
            
            return $datatable->make(true);
        }
        return view('admin.blog.index');
    }

    public function create()
    {
        $blog_categories = BlogCategory::Active()->orderBy('created_at', 'DESC')->get();        
        return view('admin.blog.create',compact('blog_categories'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //dd($request->all());
        $this->validate($request, [
            'blog_category_id' => 'required',
            'title' => 'required|string',    		
            'description' => 'required', 
            'image' => 'required|image|mimes:jpeg,png,jpg|max:50480',   		
        ]);
        
        $input = $request->except(['_token']);

        $dublicat_blog = Blog::where('blog_category_id','=',$request->blog_category_id)->where('title','=',$request->title)->first();

        if(!empty($dublicat_blog)){
            return Redirect::route('admin.blogs.index')->with('error', "Blog (".$request->title.") Already Exists. Please Enter Other Blog.");
        }
        
        if ($request->hasFile('image')) {
            // $randomSTR = randomSTR();
            // $file = $request->file('image');
            // $extension = $file->getClientOriginalName();
            // $file_name = $randomSTR . '.' . $file->getClientOriginalExtension();

            // //originals image save
            // $originalsImgPath = 'backend/assets/images/blog_images/';
            // if(!File::exists($originalsImgPath)) {
            //     File::makeDirectory($originalsImgPath, $mode = 0777, true, true);
            // }
            
            // $orignal_img = Image::make($file->getRealPath());
            // $orignal_img->save($originalsImgPath.$file_name,80);

            // //thumbnail image save
            // $thumbnailPath = 'backend/assets/images/blog_images/thumbnail/';
            // if(!File::exists($thumbnailPath)) {
            //     File::makeDirectory($thumbnailPath, $mode = 0777, true, true);
            // }
            // $thumb_img = Image::make($file->getRealPath())->fit(100, 100, function ($constraint) {
            //     $constraint->aspectRatio(); //maintain image ratio
            // });
            // $thumb_img->save($thumbnailPath.$file_name,80);

            // $input['image'] = $file_name;

            $file = $request->file('image');
            $image_name = SlugService::createSlug(Blog::class, 'slug', $request->title);
            $file_name = $image_name . '.webp';

            //originals image save
            $original_file_path = '/';
            Helper::createDirecrotoryIfNotExist('blogs', $original_file_path);
            Storage::disk('blogs')->putFileAs($original_file_path, $file, $file_name);

            //thumbnail image save
            $thumbnailPath = '/thumbnail/';
            Helper::createDirecrotoryIfNotExist('blogs', $thumbnailPath);
            $thumb_img = Image::make($file->getRealPath())->fit(100, 100, function ($constraint) {
                $constraint->aspectRatio(); //maintain image ratio
            });
            $thumb_img->stream();
            Storage::disk('blogs')->put($thumbnailPath.$file_name, $thumb_img);

            $input['image'] = $file_name;
        }
            
        return DB::transaction(function() use($input){
            Blog::Create($input);
            return Redirect::route('admin.blogs.index')->with('success','Blog created successfully.');
        });
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $blog_categories = BlogCategory::Active()->orderBy('created_at', 'DESC')->get();
        $blog = Blog::find($id);
        if(empty($blog)){
            return Redirect::route('admin.blogs.index')->with('error','Blog not found.');
        }

        return view('admin.blog.edit',compact('blog_categories', 'blog'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Blog $blog)
    {
        $this->validate($request, [
            'blog_category_id' => 'required',
            'title' => 'required',   		
            'description' => 'required',
            'image' => 'image|mimes:jpeg,png,jpg|max:50480',   		
        ]);
        
        $input = $request->except(['_token','_method']);

        $blog_id = $request->id;
    	$blog_category_id = $request->id;
        $blog_title = $request->title;

        $dublicat_blog = Blog::whereNotIn('id',[$blog_category_id])->where('blog_category_id','=',$blog_category_id)->where('title','=',$blog_title)->first();

        if(!empty($dublicat_blog)){
            return Redirect::route('admin.blogs.edit', $blog_category_id )->with('error', "Blog (".$blog_title.") Already Exists. Please Enter Other Blog.");
        }
        
        if ($request->hasFile('image')) {
            // $randomSTR = randomSTR();
            // $file = $request->file('image');
            // $extension = $file->getClientOriginalName();
            // $file_name = $randomSTR . '.' . $file->getClientOriginalExtension();

            // //delete old file
            // $originalsImgPath = 'backend/assets/images/blog_images/'.$blog->image;
            // $thumbnailPath = 'backend/assets/images/blog_images/thumbnail/'.$blog->image;
            // if(File::exists($originalsImgPath)) {
            //     File::delete($originalsImgPath);
            // }
            // if(File::exists($thumbnailPath)) {
            //     File::delete($thumbnailPath);
            // }

            // //originals image save
            // $originalsImgPath = 'backend/assets/images/blog_images/';
            // $orignal_img = Image::make($file->getRealPath());
            // $orignal_img->save($originalsImgPath.$file_name,80);

            // //thumbnail image save
            // $thumbnailPath = 'backend/assets/images/blog_images/thumbnail/';
            // if(!File::exists($thumbnailPath)) {
            //     File::makeDirectory($thumbnailPath, $mode = 0777, true, true);
            // }
            // $thumb_img = Image::make($file->getRealPath())->fit(100, 100, function ($constraint) {
            //     $constraint->aspectRatio(); //maintain image ratio
            // });
            // $thumb_img->save($thumbnailPath.$file_name,80);

            // $input['image'] = $file_name;

            $file = $request->file('image');
            $image_name = SlugService::createSlug(Blog::class, 'slug', $request->title);
            $file_name = $image_name . '.webp';

            //originals image save
            $original_file_path = '/';
            Helper::createDirecrotoryIfNotExist('blogs', $original_file_path);
            Helper::deleteFileIfExist('blogs', $original_file_path . $blog->image);
            Storage::disk('blogs')->putFileAs($original_file_path, $file, $file_name);

            //thumbnail image save
            $thumbnail_path = '/thumbnail/';
            Helper::createDirecrotoryIfNotExist('blogs', $thumbnail_path);
            Helper::deleteFileIfExist('blogs', $thumbnail_path . $blog->image);
            $img_thumbnail = Image::make($file->getRealPath())->fit(100, 100, function ($constraint) {
                $constraint->aspectRatio(); //maintain image ratio
            });
            $img_thumbnail->stream();
            Storage::disk('blogs')->put($thumbnail_path.$file_name, $img_thumbnail);

            $input['image'] = $file_name;
        }

        return DB::transaction(function() use($input, $blog){
            $blog->update($input);
            return Redirect::route('admin.blogs.index')->with('success','Blog update successfully.');
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, $id)
    {
        $blog = Blog::find($id);

        //delete product image old file
        // $originalsImgPath = 'backend/assets/images/blog_images/'.$blog->image;
        // $thumbnailPath = 'backend/assets/images/blog_images/thumbnail/'.$blog->image;
        // if(File::exists($originalsImgPath)) {
        //     File::delete($originalsImgPath);
        // }
        // if(File::exists($thumbnailPath)) {
        //     File::delete($thumbnailPath);
        // }
        $originalsImgPath = '/';
        $thumbnailPath = '/thumbnail/';
        if(Helper::checkFileExist('blogs', $originalsImgPath)) {
            Helper::deleteFileIfExist('blogs', $originalsImgPath . $blog->image);
        }
        if(Helper::checkFileExist('blogs', $thumbnailPath)) {
            Helper::deleteFileIfExist('blogs', $thumbnailPath . $blog->image);
        }

        $blog->delete();

        if ($request->ajax()) {
            return response()->json(['status' => 'success', 'msg' => "Blog deleted successfully."]);
        }
        return Redirect::route('admin.blogs.index')->with('success','Blog deleted successfully.');
    }

    public function BulkRemove(Request $request)
    {
        $ids = $request->id;
        $blogs = Blog::whereIn('id', $ids)->get();
        foreach($blogs as $blog){
            //delete product image old file
            // $originalsImgPath = 'backend/assets/images/blog_images/'.$blog->image;
            // $thumbnailPath = 'backend/assets/images/blog_images/thumbnail/'.$blog->image;
            // if(File::exists($originalsImgPath)) {
            //     File::delete($originalsImgPath);
            // }
            // if(File::exists($thumbnailPath)) {
            //     File::delete($thumbnailPath);
            // }

            $originalsImgPath = '/';
            $thumbnailPath = '/thumbnail/';
            if(Helper::checkFileExist('blogs', $originalsImgPath)) {
                Helper::deleteFileIfExist('blogs', $originalsImgPath . $blog->image);
            }
            if(Helper::checkFileExist('blogs', $thumbnailPath)) {
                Helper::deleteFileIfExist('blogs', $thumbnailPath . $blog->image);
            }

            $blog->delete();
        }
        // Category::whereIn('id',$ids)->delete();
        return response()->json(['status' => 'success', 'msg' => "Blog deleted successfully."]);
        
    }

    public function Status(Request $request){
        $blog = Blog::find($request->id);
        $blog->is_active = (int) $request->status;
        $blog->save();
        return response()->json(['status' => 'success']);
    }
}
