<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Cviebrock\EloquentSluggable\Services\SlugService;

use App\Models\Admin\Category;

use DataTables;
use Response;
use Mail;
use Redirect;
use Auth;
use Hash;
use Image;
use Helper;
use Illuminate\Validation\Rule;

class CategoryController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $raw = Category::with('parent');
            // $raw = Category::with([
            //     'parent' => function ($query) {
            //         $query->as('p');
            //     }
            // ]);

            //for category filter
            $category_id = $request->category_id;
            if ($category_id > 0) {
                $raw = $raw->where('categories.parent_id', $category_id);
            }

            //for status filter
            if ($request->has('status')) {
                $raw = $raw->where('categories.status', '=', $request->status);
            }

            $raw->get();

            $datatable = app('datatables')->of($raw)

            ->editColumn('parent.name', function ($raw) {
                if($raw->parent_id == '' || $raw->parent_id == null || $raw->parent_id == 0){
                    return '<span class="label theme-bg3 text-white f-12">Main Category</span>';
                }else{
                    if(!empty($raw->parent)){
                        // return $raw->parent->name;
                        return '<span class="label text-white f-12" style="background:linear-gradient(-135deg, #1de9b6 0%, #'.$raw->parent->color_code.' 100%);">'.$raw->parent->name.'</span>';
                    }else{
                        return '<span class="label theme-bg text-white f-12">N/A</span>';
                    }
                }
            })

            // ->editColumn('name', function ($raw) {
            //     if(!isset($raw->parent)){
            //         return '<h6 class="m-0 text-c-black"><i class="fas fa-circle text-c-green f-12"></i> '.$raw->name.'</h6>';
            //     }else{
            //         return $raw->name;
            //     }
            // })

            ->editColumn('created_at', function ($raw) {
                if($raw->created_at == null || $raw->created_at == ''){
                    return '-';
                }else{
                    return date('Y-m-d H:i:s A', strtotime($raw->created_at));
                }
            })
            ->editColumn('status', function ($raw) {
                if(in_array('admin', Auth::user()->roles->pluck('name')->toArray())){
                    if($raw->status == 1){
                        return '<span class="badge badge-success category_status active" data-url="'.route('admin.categories.status', $raw->id).'">Active</span>';
                    }else{
                        return '<span class="badge badge-dark category_status inactive" data-url="'.route('admin.categories.status', $raw->id).'">Deactive</span>';
                    }
                }
                else{
                    if($raw->status == 1){
                        return '<span class="badge badge-success">Active</span>';
                    }else{
                        return '<span class="badge badge-dark">Deactive</span>';
                    }
                }
            })
            ->addColumn('action', function ($raw) {
                $html = '';
                if(auth()->user()->can('edit_categories')){
                    $html .= '<a href="'.route("admin.categories.edit", $raw->id).'" class="label theme-bg2 text-white f-12">Edit</a>';
                }
                if(auth()->user()->can('delete_categories')){
                    $html .= '<a href="javascript:void(0)" class="label theme-bg text-white f-12 delete-category-btn" data-url="'. route("admin.categories.destroy", $raw->id) .'">Delete</a>';
                }
                return $html;
            });
            return $datatable->make(true);
        }
        $categories = Category::Active()->where('parent_id', null)->orderby('name', 'asc')->get();
        return view('admin.category.index', compact('categories'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $categories = Category::Active()->where('parent_id', null)->orderby('name', 'asc')->get();
        return view('admin.category.create', compact('categories'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'category_id' => 'nullable|numeric',
            // 'name' => 'required|min:2|unique:categories,name,NULL,id,deleted_at,NULL',
            'name' => [
                'required',
                Rule::unique('categories')->where(function ($query) use($request) {
                    return $query->where('name', $request->name)
                    ->where('parent_id', $request->parent_id);
                }),
            ],
        ],[
            'name.unique' => 'Given subcategory already exist in parent category',
        ]);

        $input = $request->only('category_id', 'name', 'meta_title', 'meta_author', 'meta_keyword', 'meta_description');
        $input['parent_id'] = $request->category_id;
        if($request->category_id == '' || $request->category_id == null)
            $input['color_code'] = Helper::random_color();

        // dd($input);
        if(Category::create($input)) {
            return redirect()->route('admin.categories.index')->with('success','Category create successfully.');
        }


        return redirect()->back()->with('error','Unable to create category.');
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Category $category)
    {
        if(empty($category)){
            return redirect()->route('admin.categories.index')->with('error', 'Category not found.');
        }
        $categories = Category::Active()->where('parent_id', null)->orderby('name', 'asc')->get();
        return view('admin.category.edit', compact('category', 'categories'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Category $category)
    {
        $this->validate($request, [
            // 'category_id' => 'required',
            'name' => [
                'required',
                Rule::unique('categories')->where(function ($query) use($request, $category) {
                    return $query->where('name', $request->name)
                    ->where('parent_id', $request->parent_id)
                    ->whereNotIn('id', [$category->id]);
                }),
            ],
        ],[
            'name.unique' => 'Given subcategory already exist in parent category',
        ]);

        $input = $request->only('category_id', 'name', 'meta_title', 'meta_author', 'meta_keyword', 'meta_description');

        if($request->name != $category->name){
            $input['slug'] = SlugService::createSlug(Category::class, 'slug', $request->name);
        }

        $input['parent_id'] = $request->category_id;

        if($category->update($input) ) {
            return redirect()->route('admin.categories.index')->with('success','Category update successfully.');
        }

        return redirect()->back()->with('error','Unable to update category.');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Request $request, Category $category)
    {
        //if not found
        if(empty($category)){
            if ($request->ajax()) {
                return response()->json(['status' => 'error', 'msg' => "Some problem delete category."]);
            }
            return Redirect::route('admin.categories.index')->with('error','Some problem delete category.');
        }

        //delete
        $category->delete();
        if ($request->ajax()) {
            return response()->json(['status' => 'success', 'msg' => "Category Deleted successfully."]);
        }
        return Redirect::route('admin.categories.index')->with('success','Category delete successfully.');
    }

    public function BulkRemove(Request $request)
    {
        $ids = $request->id;
        $categories = Category::whereIn('id',$ids)->get();
        if($categories->isEmpty()){
            return response()->json(['status' => 'error', 'msg' => "Some problem delete categories."]);
        }
        Category::whereIn('id',$ids)->delete();
        return response()->json(['status' => 'success', 'msg' => "Categories deleted successfully."]);
    }

    public function Status(Request $request){
        $category = Category::find($request->id);
        $category->status = $request->status;
        $category->save();
        return response()->json(['status' => 'success']);
    }
}
