<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Admin\LandingPage;

use DataTables;
use Response;
use Mail;
use Excel;
use Redirect;
use Image;
use File;
use Storage;
use DB;

use App\Traits\Authorizable;

class LandingPageController extends Controller
{
    use Authorizable;
    
    public function __construct() {
        $this->middleware('auth:admin');
        //$this->middleware('admin');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $landing_page = LandingPage::orderBy('created_at', 'desc')->first();
        return view('admin.landing_page.index', compact('landing_page'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // dd($request->all());
        $this->validate($request, [
            'page1_title' => 'required',
    		'page1_content' => 'required',
    		'page2_title' => 'required',
    		'page2_content' => 'required',
            'page3_title' => 'required',
            'page3_content' => 'required',
        ]);

        $input = $request->except(['_token']);
        $input['is_active'] = (int) $request->is_active;
        return DB::transaction(function() use($input){
            LandingPage::Create($input);
            return Redirect::route('admin.landing-pages.index')->with('success','Landing Page create successfully.');
        });
        
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $landingpage = LandingPage::find($id);
        // dd($landingpage);
        $this->validate($request, [
            'page1_title' => 'required',
    		'page1_content' => 'required',
    		'page2_title' => 'required',
    		'page2_content' => 'required',
            'page3_title' => 'required',
            'page3_content' => 'required',
        ]);

        $input = $request->except(['_token']);
        $input['is_active'] = (int) $request->is_active;
        return DB::transaction(function() use($input, $landingpage){
            $landingpage->update($input);
            return Redirect::route('admin.landing-pages.index')->with('success','Landing Page update successfully.');
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
