<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Admin\Logo;

use DataTables;
use Response;
use Mail;
use Excel;
use Redirect;
use Image;
use File;
use Storage;
use DB;
use Helper;

use \Cviebrock\EloquentSluggable\Services\SlugService;

use App\Traits\Authorizable;

class LogoController extends Controller
{

    use Authorizable;
    
    public function __construct() {
        $this->middleware('auth:admin');
        //$this->middleware('admin');
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $logo = Logo::first();
        return view('admin.logo.index',compact('logo'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'caption' => 'required',
    		'logo_image' => 'required',
        ]);

        $input = $request->except(['_token']);

        if ($request->hasFile('logo_image')) {
            // $randomSTR = randomSTR();
            // $file = $request->file('logo_image');
            // $extension = $file->getClientOriginalName();
            // $file_name = $randomSTR . '.' . $file->getClientOriginalExtension();

            // //originals image save
            // $originalsImgPath = 'backend/assets/images/logo_image/';
            // if(!File::exists($originalsImgPath)) {
            //     File::makeDirectory($originalsImgPath, $mode = 0777, true, true);
            // }
            // $orignal_img = Image::make($file->getRealPath());
            // $orignal_img->save($originalsImgPath.$file_name,80);

            // //thumbnail image save
            // $thumbnailPath = 'backend/assets/images/logo_image/thumbnail/';
            // if(!File::exists($thumbnailPath)) {
            //     File::makeDirectory($thumbnailPath, $mode = 0777, true, true);
            // }
            // // $thumb_img = Image::make($file->getRealPath())->resize(100, 100);
            // $thumb_img = Image::make($file->getRealPath())->fit(100, 100, function ($constraint) {
            //     $constraint->aspectRatio(); //maintain image ratio
            // });
            // $thumb_img->save($thumbnailPath.$file_name,80);

            // $input['logo_image'] = $file_name;

            $file = $request->file('logo_image');
            $image_name = SlugService::createSlug(Logo::class, 'slug', $request->caption);
            $file_name = $image_name . '.webp';

            //originals image save
            $original_file_path = '/';
            Helper::createDirecrotoryIfNotExist('logos', $original_file_path);
            Storage::disk('logos')->putFileAs($original_file_path, $file, $file_name);

            //thumbnail image save
            $thumbnailPath = '/thumbnail/';
            Helper::createDirecrotoryIfNotExist('logos', $thumbnailPath);
            $thumb_img = Image::make($file->getRealPath())->fit(100, 100, function ($constraint) {
                $constraint->aspectRatio(); //maintain image ratio
            });
            $thumb_img->stream();
            Storage::disk('logos')->put($thumbnailPath.$file_name, $thumb_img);

            $input['logo_image'] = $file_name;
        }
        return DB::transaction(function() use($input){
            Logo::Create($input);
            return Redirect::route('admin.logos.index')->with('success', 'Logo create successfully.');
        });
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Logo $logo)
    {
        // $logo = Logo::find($id);

        $this->validate($request, [
            'caption' => 'required',    		
        ]);

        $input = $request->except(['_token','_method']);

        if ($request->hasFile('logo_image')) {
            // $randomSTR = randomSTR();
            // $file = $request->file('logo_image');
            // $extension = $file->getClientOriginalName();
            // $file_name = $randomSTR . '.' . $file->getClientOriginalExtension();

            // //delete old file
            // $originalsImgPath = 'backend/assets/images/logo_image/'.$logo->logo_image;
            // $thumbnailPath = 'backend/assets/images/logo_image/thumbnail/'.$logo->logo_image;
            // if(File::exists($originalsImgPath)) {
            //     File::delete($originalsImgPath);
            // }
            // if(File::exists($thumbnailPath)) {
            //     File::delete($thumbnailPath);
            // }

            // //originals image save
            // $originalsImgPath = 'backend/assets/images/logo_image/';
            // if(!File::exists($originalsImgPath)) {
            //     File::makeDirectory($originalsImgPath, $mode = 0777, true, true);
            // }
            // $orignal_img = Image::make($file->getRealPath());
            // $orignal_img->save($originalsImgPath.$file_name,80);

            // //thumbnail image save
            // $thumbnailPath = 'backend/assets/images/logo_image/thumbnail/';
            // if(!File::exists($thumbnailPath)) {
            //     File::makeDirectory($thumbnailPath, $mode = 0777, true, true);
            // }
            // // $thumb_img = Image::make($file->getRealPath())->resize(100, 100);
            // $thumb_img = Image::make($file->getRealPath())->fit(100, 100, function ($constraint) {
            //     $constraint->aspectRatio(); //maintain image ratio
            // });
            // $thumb_img->save($thumbnailPath.$file_name,80);

            // $input['logo_image'] = $file_name;

            $file = $request->file('logo_image');
            $image_name = SlugService::createSlug(Logo::class, 'slug', $request->caption);
            $file_name = $image_name . '.webp';

            //originals image save
            $original_file_path = '/';
            Helper::createDirecrotoryIfNotExist('logos', $original_file_path);
            Helper::deleteFileIfExist('logos', $original_file_path . $logo->logo_image);
            Storage::disk('logos')->putFileAs($original_file_path, $file, $file_name);

            //thumbnail image save
            $thumbnail_path = '/thumbnail/';
            Helper::createDirecrotoryIfNotExist('logos', $thumbnail_path);
            Helper::deleteFileIfExist('logos', $thumbnail_path . $logo->logo_image);
            $img_thumbnail = Image::make($file->getRealPath())->fit(50, 50, function ($constraint) {
                $constraint->aspectRatio(); //maintain image ratio
            });
            $img_thumbnail->stream();
            Storage::disk('logos')->put($thumbnail_path.$file_name, $img_thumbnail);

            $input['logo_image'] = $file_name;
        }

        return DB::transaction(function() use($input, $logo){
            $logo->update($input);
            return Redirect::route('admin.logos.index')->with('success', 'Logo update successfully.');
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
