<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Admin\Order;
use App\Models\Admin\Product;

use Auth;
use DataTables;
use Response;
use Mail;
use Excel;
use Redirect;
use Image;
use File;
use Storage;
use DB;
use Helper;
use Carbon\Carbon;
use Illuminate\Validation\Rule;

use \Cviebrock\EloquentSluggable\Services\SlugService;

use App\Traits\Authorizable;

class OrderController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $raw = Order::orderBy('created_at', 'desc');

            //for status filter
            if ($request->has('status')) {
                $raw->where('orders.status', '=', $request->status);
            }

            $datatable = app('datatables')->of($raw)

            ->editColumn('created_at', function ($raw) {
                if($raw->created_at == ''){
                    return '-';
                }else{
                    // return date('Y-m-d H:i:s A', strtotime($raw->created_at));
                    // return date('d F Y', strtotime($raw->created_at));
                    return $raw->created_at;
                }
            })

            ->editColumn('grand_total', function ($raw) {
                return Helper::getCurrency().' '.$raw->grand_total;
            })

            ->addColumn('orderer_name', function ($raw) {    
                if($raw->user_id == null){
                    $billing_address = $raw->billing_address;
                    return $billing_address->fname.' '.$billing_address->lname;
                }else{
                    // return $raw->user->first_name.' '.$raw->user->last_name;
                    return $raw->user->first_name;
                }              
            })

            ->editColumn('status', function ($raw) {
                if($raw->status == 'success'){
                    return '<span class="badge badge-pill badge-success">'.ucwords($raw->status).'</span>';
                }elseif($raw->status == 'pending'){
                    return '<span class="badge badge-pill badge-warning">'.ucwords($raw->status).'</span>';
                }else{
                    return '<span class="badge badge-pill badge-danger">'.ucwords($raw->status).'</span>';
                }
            })

            ->addColumn('action', function ($raw) {
                $html = '';
                if(auth()->user()->can('view_orders')){
                    $html .= '<a href="'.route("admin.orders.show", $raw->id).'" class="label theme-bg2 text-white f-12">View</a>  ';
                }
                if(auth()->user()->can('delete_orders')){
                    $html .= '<a class="label theme-bg text-white f-12 delete-order-btn" href="javascript:void(0)" data-url="'. route("admin.orders.destroy", $raw->id) .'">Delete</a>';
                }
                return $html;
            });

            return $datatable->make(true);
        }

        return view('admin.order.index');
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        $order = Order::findOrFail($id);
        $order_details = $order->order_details;
        $transaction = $order->transaction;
        return view('admin.order.show', compact('order', 'order_details', 'transaction'));
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        $order = Order::findOrFail($id);
        $order->delete();

        if ($request->ajax())
            return response()->json(['status' => 'success', 'msg' => "Order deleted successfully."]);

        return Redirect::route('admin.orders.index')->with('success','Order deleted successfully.');
    }
}
