<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Admin\Page;

use DataTables;
use Response;
use Mail;
use Excel;
use Redirect;
use Image;
use File;
use Storage;
use DB;

use App\Traits\Authorizable;

class PageController extends Controller
{
    use Authorizable;
    
    public function __construct() {
        $this->middleware('auth:admin');
        //$this->middleware('admin');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $raw = Page::query();
            $datatable = app('datatables')->of($raw)
            ->editColumn('created_at', function ($raw) {
                if($raw->created_at == ''){
                    return '-';
                }else{
                    return date('Y-m-d H:i:s A', strtotime($raw->created_at));
                }
            })
    
            ->editColumn('description', function ($raw) {
                if($raw->description == ''){
                    return '-';
                }else{
                    if(strlen($raw->description) > 50){
                        return substr($raw->description,0,49).'...';
                    }else{
                        return $raw->description;
                    }
                    // return (strlen($raw->description) > 50) ? substr($raw->description,0,49).'...' : $raw->description;
                }
            })
    
            ->editColumn('is_active', function ($raw) {
                if($raw->is_active == '1'){
                    return '<span class="badge badge-success page_status active" data-url="'.route('admin.pages.status', $raw->id).'">Active</span>';
                }else{
                    return '<span class="badge badge-dark page_status inactive" data-url="'.route('admin.pages.status', $raw->id).'">Deactive</span>';
                }
            })
    
            ->addColumn('action', function ($raw) {
                $html = '';
                if(auth()->user()->can('edit_pages')){
                    $html .= '<a href="'.route("admin.pages.edit", $raw->id).'" class="label theme-bg2 text-white f-12">Edit</a>  ';
                }
                if(auth()->user()->can('delete_pages')){
                    $html .= '<a class="label theme-bg text-white f-12 delete-page-btn" href="javascript:void(0)" data-url="'. route("admin.pages.destroy", $raw->id) .'">Delete</a>';
                }
                return $html;
            });
            
            return $datatable->make(true);
        }
        return view('admin.page.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.page.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // dd($request->all());
        $this->validate($request, [
            'page_name' => 'required|string||unique:pages',
    		'description' => 'required',
    		'meta_title' => 'required',
    		'meta_description' => 'required',
    		'meta_keywords' => 'required',
        ]);

        $input = $request->except(['_token']);
        $input['slug'] = str_slug($request->page_name, '-');

        return DB::transaction(function() use($input){
            Page::Create($input);
            return Redirect::route('admin.pages.index')->with('success', 'Page create successfully.');
        });
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $page = Page::find($id);
        if(empty($page)){
            return Redirect::route('admin.pages.index')->with('error','Page not found.');
        }
        return view('admin.page.edit',compact('page'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Page $page)
    {
        $this->validate($request, [
            'page_name' => 'required|string|unique:pages,page_name,'.$page->id,
    		'description' => 'required',
    		'meta_title' => 'required',
    		'meta_description' => 'required',
    		'meta_keywords' => 'required',
        ]);

        $input = $request->except(['_token','_method']);
        $input['slug'] = str_slug($request->page_name, '-');
        return DB::transaction(function() use($input, $page){
            $page->update($input);
            return Redirect::route('admin.pages.index')->with('success','Page update successfully.');
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, $id)
    {
        $page = Page::find($id);

        if(empty($page)){
            if ($request->ajax()) {
                return response()->json(['status' => 'error', 'msg' => "Page not found."]);
            }
            return Redirect::route('admin.pages.index')->with('error','Page not found.');
        }

        $page->delete();
        if ($request->ajax()) {
            return response()->json(['status' => 'success', 'msg' => "Page deleted successfully."]);
        }
        return Redirect::route('admin.pages.index')->with('success','Page deleted successfully.');
        
    }

    public function BulkRemove(Request $request)
    {
        $ids = $request->id;
        Page::whereIn('id',$ids)->delete();
        return response()->json(['status' => 'success', 'msg' => "Page deleted successfully."]);
        
    }

    public function Status(Request $request){
        $page = Page::find($request->id);
        $page->is_active = (int)$request->status;
        $page->save();
        return response()->json(['status' => 'success']);
    }
}
