<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Admin\Permission;
use App\Models\Admin\Role;

use DataTables;
use Response;
use Mail;
use Excel;
use Redirect;
use Image;
use File;
use Storage;
use DB;

use App\Traits\Authorizable;

class PermissionController extends Controller
{
    use Authorizable;
    
    public function __construct() {
        $this->middleware('auth:admin');
        //$this->middleware('admin');
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $raw = Permission::query();
            $raw->where('guard_name', 'admin');
    
            $datatable = app('datatables')->of($raw)
            ->editColumn('created_at', function ($raw) {
                if($raw->created_at == ''){
                    return '-';
                }else{
                    return date('Y-m-d H:i:s A', strtotime($raw->created_at));
                }
            })
            ->addColumn('action', function ($raw) {
                $html = '';
                if(auth()->user()->can('edit_permissions')){
                    $html .= '<a href="'.route("admin.permissions.edit", $raw->id).'" class="label theme-bg2 text-white f-12">Edit</a>';
                }
                if(auth()->user()->can('delete_permissions')){
                    $html .= '<a href="javascript:void(0)" class="label theme-bg text-white f-12 delete-permission-btn" data-url="'. route("admin.permissions.destroy", $raw->id) .'">Delete</a>';
                }
                return $html;
            });
            return $datatable->make(true);
        }
        return view('admin.permission.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.permission.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name' => 'required|unique:permissions',
        ]);

        //add new permission
        return DB::transaction(function() use($request){
            Permission::firstOrCreate(['name' => $request->name, 'guard_name' => 'admin']);
            //give permission to admin role
            $role = Role::where('name', 'admin')->first();
            $role->givePermissionTo($request->name);

            return Redirect::route('admin.permissions.index')->with('success','Permission create successfully.');
        });
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $permission = Permission::find($id);
        return view('admin.permission.edit',compact('permission'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $permission = permission::find($id);
        $this->validate($request, [
            'name' => 'required|unique:permissions,name,'.$permission->id,
        ]);

        $input = $request->except(['_token','_method']);
        return DB::transaction(function() use($input, $permission){
            $permission->update($input);
            return Redirect::route('admin.permissions.index')->with('success','Permission update successfully.');
        });
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, $id)
    {
        $permission = Permission::find($id);
        //if not found
        if(empty($permission)){
            if ($request->ajax()) {
                return response()->json(['status' => 'error', 'msg' => "Some problem delete permission."]);    
            }
            return Redirect::route('admin.permissions.index')->with('error','Some problem delete permission.');
        }

        //delete
        $permission->delete();
        if ($request->ajax()) {
            return response()->json(['status' => 'success', 'msg' => "Permission Deleted successfully."]);
        }
        return Redirect::route('admin.permissions.index')->with('success','Permission delete successfully.');
    }

    public function BulkRemove(Request $request)
    {
        $ids = $request->id;
        $permissions = Permission::whereIn('id',$ids)->get();
        if($permissions->isEmpty()){
            return response()->json(['status' => 'error', 'msg' => "Some problem delete permission."]);
        }
        // echo '<pre>';print_r($permissions);exit;
        Permission::whereIn('id',$ids)->delete();
        return response()->json(['status' => 'success', 'msg' => "Permission deleted successfully."]);
    }

}
