<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Admin\Category;
use App\Models\Admin\Product;
use App\Models\Admin\ProductImage;

use Auth;
use DataTables;
use Response;
use Mail;
use Excel;
use Redirect;
use Image;
use File;
use Storage;
use DB;
use Helper;
use Carbon\Carbon;
use Illuminate\Validation\Rule;

use \Cviebrock\EloquentSluggable\Services\SlugService;

use App\Traits\Authorizable;

class ProductController extends Controller
{
    use Authorizable;

    // public function __construct() {
    //     $this->middleware('auth:admin');
    //     //$this->middleware('admin');
    // }

    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $raw = Product::with(['category']);

            //for category filter
            $category_id = $request->category_id;
            if ($category_id > 0) {
                $raw->where('products.category_id', $category_id);
            }

            //for status filter
            if ($request->has('status')) {
                $raw->where('products.is_active', '=', $request->status);
            }

            $datatable = app('datatables')->of($raw)

            ->editColumn('created_at', function ($raw) {
                if($raw->created_at == ''){
                    return '-';
                }else{
                    // return date('Y-m-d H:i:s A', strtotime($raw->created_at));
                    return date('d F Y', strtotime($raw->created_at));
                }
            })

            // ->editColumn('product_image', function ($raw) {
            //     if($raw->product_image == '' || $raw->product_image == null){
            //         return '-';
            //     }else{
            //     // return '<img src="'.asset('/backend/assets/images/product_images/thumbnail/').'/'.$raw->product_image.'" width="50">';
            //     return '<a class="example-image-link" href="'.asset('backend/assets/images/product_images/').'/'.$raw->product_image.'" data-lightbox="example-1"><img class="example-image" src="'.asset('backend/assets/images/product_images/thumbnail/').'/'.$raw->product_image.'" width="50"></a>';
            //     }
            // })

            ->editColumn('category_id', function ($raw) {
                if($raw->category_id == '' || $raw->category_id == null || $raw->category_id == 0){
                    return 'N/A';
                }else{
                    if(!empty($raw->category->name)){
                        return $raw->category->name;
                    }else{
                        return 'N/A';
                    }
                }
            })

            ->editColumn('is_active', function ($raw) {
                if(in_array('admin', Auth::user()->roles->pluck('name')->toArray())){
                    if($raw->is_active == 1){
                        return '<span class="badge badge-success product_status active" data-url="'.route('admin.products.status', $raw->id).'">Active</span>';
                    }else{
                        return '<span class="badge badge-dark product_status inactive" data-url="'.route('admin.products.status', $raw->id).'">Deactive</span>';
                    }
                }
                else{
                    if($raw->is_active == 1){
                        return '<span class="badge badge-success">Active</span>';
                    }else{
                        return '<span class="badge badge-dark">Deactive</span>';
                    }
                }
            })

            ->addColumn('action', function ($raw) {
                $html = '';
                if(auth()->user()->can('edit_products')){
                    $html .= '<a href="'.route("admin.products.edit", $raw->id).'" class="label theme-bg2 text-white f-12">Edit</a>  ';
                }
                if(auth()->user()->can('delete_products')){
                    $html .= '<a class="label theme-bg text-white f-12 delete-product-btn" href="javascript:void(0)" data-url="'. route("admin.products.destroy", $raw->id) .'">Delete</a>';
                }
                return $html;
            });

            return $datatable->make(true);
        }

        $categories = Category::Active()->where('parent_id','<>', null)->orderby('name', 'asc')->get();
        return view('admin.product.index', compact('categories'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $categories = Category::Active()->where('parent_id','<>', null)->orderby('name', 'asc')->get();
        return view('admin.product.create', compact('categories'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'category_id' => 'required',
            'name' => 'required|string',
    		'description' => 'required',
            'name' => [
                'required',
                Rule::unique('products')->where(function ($query) use($request) {
                    return $query->where('category_id', $request->category_id)
                    ->where('name', $request->name);
                }),
            ],
    	],[
            'name.unique' => 'Given name already exist in this category',
        ]);
        // dd($request->all());

        // dd(date('hisU'));

        $input = $request->only(['category_id', 'name', 'old_price', 'description']);
        $input['admin_id'] = Auth::id();
        $input['price'] = $request->default_price;

        $attribute_value_ids_array = $request->attribute_value_ids;
        $price_array = $request->price;
        $quantity_array = $request->quantity;

        if(!empty($attribute_value_ids_array) && !empty($price_array) && !empty($quantity_array)){
            $product_attributes_array = [];
            foreach($price_array as $key => $price){
                $single_array = [];
                foreach($attribute_value_ids_array as $attribute_id => $attribute_values){
                    $single_array[$attribute_id] = $attribute_values[$key];
                }
                $single_array['price'] = $price;
                $single_array['quantity'] = $quantity_array[$key];

                $product_attributes_array[] = $single_array;
            }

            // dump($product_attributes_array);exit;
            $product_attributes_array = Helper::searchAndRemoveDuplicateArray($product_attributes_array, 'color', 'size');
            // dump($product_attributes_array);

            $input['add_edit_manage_product_attributes'] = $product_attributes_array;
            $input['product_attributes'] = Helper::groupArray($product_attributes_array, "color", true);
            $input['quantity'] = array_sum(array_column($product_attributes_array, 'quantity'));
        }

        DB::transaction(function() use($input, $request){
            $product = Product::Create($input);

            //Save product images
            if($request->has('product_images')){
                $product_images_array = $request->product_images;
                if(!empty($product_images_array)){
                    $product_images_insert_array = [];
                    // $i = 1;
                    foreach($product_images_array as $file){
                        $temp_file_path = '/tmp/uploads/' . $file;
                        if(Helper::checkFileExist('images', $temp_file_path)){
                            // $image_name = SlugService::createSlug(Product::class, 'slug', $request->name);
                            $image_name = $product->slug;
                            $temp_file_full_path = Storage::disk('images')->url($temp_file_path);

                            // Original Image
                            $file_name = $image_name . '_' . date('hisU') . '.webp';

                            //80x80 image save
                            $file_80_80_path = '/80_80/';
                            Helper::createDirecrotoryIfNotExist('products', $file_80_80_path);
                            $img_80_80 = Image::make($temp_file_full_path)->fit(80, 80, function ($constraint) {
                                $constraint->aspectRatio(); //maintain image ratio
                            });
                            $img_80_80->stream();
                            Storage::disk('products')->put($file_80_80_path.$file_name, $img_80_80);

                            //150x160 image save
                            $file_150_160_path = '/150_160/';
                            Helper::createDirecrotoryIfNotExist('products', $file_150_160_path);
                            $img_150_160 = Image::make($temp_file_full_path)->fit(150, 160, function ($constraint) {
                                $constraint->aspectRatio(); //maintain image ratio
                            });
                            $img_150_160->stream();
                            Storage::disk('products')->put($file_150_160_path.$file_name, $img_150_160);


                            //540x600 image save
                            $file_540_600_path = '/540_600/';
                            Helper::createDirecrotoryIfNotExist('products', $file_540_600_path);
                            $img_540_600 = Image::make($temp_file_full_path)->fit(540, 600, function ($constraint) {
                                $constraint->aspectRatio(); //maintain image ratio
                            });
                            $img_540_600->stream();
                            Storage::disk('products')->put($file_540_600_path.$file_name, $img_540_600);

                            //810x900 image save
                            $file_810_900_path = '/810_900/';
                            Helper::createDirecrotoryIfNotExist('products', $file_810_900_path);
                            $img_810_900 = Image::make($temp_file_full_path)->fit(810, 900, function ($constraint) {
                                $constraint->aspectRatio(); //maintain image ratio
                            });
                            $img_810_900->stream();
                            Storage::disk('products')->put($file_810_900_path.$file_name, $img_810_900);

                            // Original Image
                            Helper::createDirecrotoryIfNotExist('products', '/');
                            File::move(storage_path('app/public/images' . $temp_file_path), storage_path('app/public/products/'.$file_name));

                            $product_images_insert_array[] = [
                                'product_id' => $product->id,
                                'file_name' => $file_name,
                                'created_at' => Carbon::now(),
                                'updated_at' => Carbon::now(),
                            ];
                            // $i++;
                        }
                    }
                    ProductImage::insert($product_images_insert_array);
                }
            }
        });
        return Redirect::route('admin.products.index')->with('success', 'Product create successfully.');
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        $product = Product::findOrFail($id);
        // if(empty($product)){
        //     return Redirect::route('admin.products.index')->with('error','Product not found.');
        // }

        // dd($product->add_edit_manage_product_attributes);

        $categories = Category::Active()->where('parent_id','<>', null)->orderby('name', 'asc')->get();
        return view('admin.product.edit',compact('categories', 'product'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        $product = Product::findOrFail($id);

        $this->validate($request, [
            'category_id' => 'required',
            'name' => 'required|string',
    		'description' => 'required',
            'name' => [
                'required',
                Rule::unique('products')->where(function ($query) use($request, $product) {
                    return $query->where('category_id', $request->category_id)
                    ->where('name', $request->name)
                    ->whereNotIn('id', [$product->id]);
                }),
            ],
    	],[
            'name.unique' => 'Given name already exist in this category',
        ]);
        // dump($request->all());
        // exit;

        $input = $request->only(['category_id', 'name', 'old_price', 'description']);
        $input['admin_id'] = Auth::id();
        $input['price'] = $request->default_price;

        $attribute_value_ids_array = $request->attribute_value_ids;
        $price_array = $request->price;
        $quantity_array = $request->quantity;

        if(!empty($attribute_value_ids_array) && !empty($price_array) && !empty($quantity_array)){
            $product_attributes_array = [];
            foreach($price_array as $key => $price){
                $single_array = [];
                foreach($attribute_value_ids_array as $attribute_id => $attribute_values){
                    $single_array[$attribute_id] = $attribute_values[$key];
                }
                $single_array['price'] = $price;
                $single_array['quantity'] = $quantity_array[$key];

                $product_attributes_array[] = $single_array;
            }

            $product_attributes_array = Helper::searchAndRemoveDuplicateArray($product_attributes_array, 'color', 'size');

            $input['add_edit_manage_product_attributes'] = $product_attributes_array;
            $input['product_attributes'] = Helper::groupArray($product_attributes_array, "color", true);
            $input['quantity'] = array_sum(array_column($product_attributes_array, 'quantity'));
        }
        // dd($input);
        return DB::transaction(function() use($input, $product, $request){
            $product->update($input);

            //Save product images
            if($request->has('product_images')){
                $product_images_array = $request->product_images;
                if(!empty($product_images_array)){
                    $product_images_insert_array = [];
                    // $i = 1;
                    foreach($product_images_array as $file){
                        $temp_file_path = '/tmp/uploads/' . $file;
                        if(Helper::checkFileExist('images', $temp_file_path)){
                            // $image_name = SlugService::createSlug(Product::class, 'slug', $request->name);
                            $image_name = $product->slug;
                            $temp_file_full_path = Storage::disk('images')->url($temp_file_path);

                            // Original Image
                            $file_name = $image_name . '_' . date('hisU') . '.webp';

                            //80x80 image save
                            $file_80_80_path = '/80_80/';
                            Helper::createDirecrotoryIfNotExist('products', $file_80_80_path);
                            $img_80_80 = Image::make($temp_file_full_path)->fit(80, 80, function ($constraint) {
                                $constraint->aspectRatio(); //maintain image ratio
                            });
                            $img_80_80->stream();
                            Storage::disk('products')->put($file_80_80_path.$file_name, $img_80_80);

                            //150x160 image save
                            $file_150_160_path = '/150_160/';
                            Helper::createDirecrotoryIfNotExist('products', $file_150_160_path);
                            $img_150_160 = Image::make($temp_file_full_path)->fit(150, 160, function ($constraint) {
                                $constraint->aspectRatio(); //maintain image ratio
                            });
                            $img_150_160->stream();
                            Storage::disk('products')->put($file_150_160_path.$file_name, $img_150_160);


                            //540x600 image save
                            $file_540_600_path = '/540_600/';
                            Helper::createDirecrotoryIfNotExist('products', $file_540_600_path);
                            $img_540_600 = Image::make($temp_file_full_path)->fit(540, 600, function ($constraint) {
                                $constraint->aspectRatio(); //maintain image ratio
                            });
                            $img_540_600->stream();
                            Storage::disk('products')->put($file_540_600_path.$file_name, $img_540_600);

                            //810x900 image save
                            $file_810_900_path = '/810_900/';
                            Helper::createDirecrotoryIfNotExist('products', $file_810_900_path);
                            $img_810_900 = Image::make($temp_file_full_path)->fit(810, 900, function ($constraint) {
                                $constraint->aspectRatio(); //maintain image ratio
                            });
                            $img_810_900->stream();
                            Storage::disk('products')->put($file_810_900_path.$file_name, $img_810_900);

                            // Original Image
                            Helper::createDirecrotoryIfNotExist('products', '/');
                            File::move(storage_path('app/public/images' . $temp_file_path), storage_path('app/public/products/'.$file_name));

                            $product_images_insert_array[] = [
                                'product_id' => $product->id,
                                'file_name' => $file_name,
                                'created_at' => Carbon::now(),
                                'updated_at' => Carbon::now(),
                            ];
                            // $i++;
                        }
                    }
                    ProductImage::insert($product_images_insert_array);
                }
            }

            return Redirect::route('admin.products.index')->with('success', 'Product update successfully.');
        });
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Request $request, string $id)
    {
        $product = Product::findOrFail($id);

        // INFO: Delete product images
        foreach($product->product_images as $image){
            $file_name = $image->file_name;

            // INFO: Delete original file
            $file_path = $file_name;
            Helper::deleteFileIfExist('products', $file_path);

            // INFO: Delete 80x80 file
            $file_path = '/80_80/'.$file_name;
            Helper::deleteFileIfExist('products', $file_path);

            // INFO: Delete 150_160 file
            $file_path = '/150_160/'.$file_name;
            Helper::deleteFileIfExist('products', $file_path);

            // INFO: Delete 540_600 file
            $file_path = '/540_600/'.$file_name;
            Helper::deleteFileIfExist('products', $file_path);

            // INFO: Delete 810_900 file
            $file_path = '/810_900/'.$file_name;
            Helper::deleteFileIfExist('products', $file_path);
        }

        $product->delete();

        if ($request->ajax())
            return response()->json(['status' => 'success', 'msg' => "Product deleted successfully."]);

        return Redirect::route('admin.products.index')->with('success','Product deleted successfully.');
    }

    public function Status(Request $request){
        $product = Product::find($request->id);
        $product->is_active = (int) $request->status;
        $product->save();
        return response()->json(['status' => 'success']);
    }

    public function storeMedia(Request $request)
    {
        $path = '/tmp/uploads/';
        Helper::createDirecrotoryIfNotExist('images', $path);

        $file = $request->file('file');

        $file_name = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
        $extension = pathinfo($file->getClientOriginalName(), PATHINFO_EXTENSION);
        $name = Helper::sanitize_file_name($file_name) . "." . $extension;

        Storage::disk('images')->putFileAs($path, $file, $name);

        return response()->json([
            'name'          => $name,
            'original_name' => $file->getClientOriginalName(),
        ]);
    }

    public function removeMedia(Request $request)
    {
        if(!$request->has('file_name'))
            return response()->json([ 'statuss' => 'error', 'message' => 'Something went wrong.']);

        $file_name = $request->file_name;

        // INFO: Delete temp folder file
        $file_path = '/tmp/uploads/' . $file_name;
        Helper::deleteFileIfExist('images', $file_path);

        // INFO: Delete original file
        $file_path = $file_name;
        Helper::deleteFileIfExist('products', $file_path);

        // INFO: Delete 80x80 file
        $file_path = '/80_80/'.$file_name;
        Helper::deleteFileIfExist('products', $file_path);

        // INFO: Delete 150_160 file
        $file_path = '/150_160/'.$file_name;
        Helper::deleteFileIfExist('products', $file_path);

        // INFO: Delete 540_600 file
        $file_path = '/540_600/'.$file_name;
        Helper::deleteFileIfExist('products', $file_path);

        // INFO: Delete 810_900 file
        $file_path = '/810_900/'.$file_name;
        Helper::deleteFileIfExist('products', $file_path);

        if($request->has('product_image_id') && $request->product_image_id != '')
            ProductImage::find($request->product_image_id)->delete();

        return response()->json(['statuss' => 'success','message' => 'File Deleted']);
    }
}
