<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Admin\Admin;
use App\Models\Admin\Permission;
use App\Models\Admin\Role;

use DataTables;
use Response;
use Mail;
use Redirect;
use Auth;
use Hash;
use Image;

use App\Traits\Authorizable;

class RoleController extends Controller
{
    use Authorizable;

    public function __construct() {
        $this->middleware('auth:admin');
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $roles = Role::where('guard_name', 'admin')->get();
        $permissions = Permission::where('guard_name', 'admin')->get();

        return view('admin.role.index', compact('roles', 'permissions'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.role.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, ['name' => 'required|unique:roles']);

        $input = $request->except('method');
        $input['guard_name'] = 'admin';

        if( Role::create($input) ) {
            return redirect()->route('admin.roles.index')->with('success','Role added successfully.');
        }

        return redirect()->back()->with('error','Unable to add role.');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        if($role = Role::findOrFail($id)) {
            // admin role has everything
            if($role->name === 'Admin' || $role->name === 'admin') {
                $role->syncPermissions(Permission::all());
                return redirect()->route('admin.roles.index');
            }

            $permissions = $request->get('permissions', []);
            $role->syncPermissions($permissions);

            return redirect()->route('admin.roles.index')->with('success', removeDash($role->name) . ' permissions has been updated');
        } else {
            return redirect()->route('admin.roles.index')->with('error', 'Role with id '. $id .' note found.');
        }

        return redirect()->route('admin.roles.index')->with('error', 'Some Problem for update permissions');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
