<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\User;
use App\Models\Admin\Permission;
use App\Models\Admin\Role;

use DataTables;
use Response;
use Mail;
use Redirect;
use Auth;
use Hash;
use Image;

use App\Traits\Authorizable;

class UserController extends Controller
{
    use Authorizable;

    public function __construct() {
        $this->middleware('auth:admin');
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return view('admin.user.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $roles = Role::where('guard_name', 'web')->pluck('name', 'id');
        return view('admin.user.create', compact('roles'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // echo 'strrrr';exit;
        $this->validate($request, [
            'name' => 'bail|required|min:2',
            'email' => 'required|email|unique:admins',
            'password' => 'required|min:6',
            'roles' => 'required|min:1'
        ]);

        // echo '<pre>';print_r($request->all());exit;

        // hash password
        $request->merge(['password' => bcrypt($request->get('password'))]);

        // Create the admin
        if ( $user = User::create($request->except('roles', 'permissions')) ) {

            assignRoleAndPermissions($request, $user);

            return redirect()->route('admin.users.index')->with('success','User has been created.');

        } else {
            return redirect()->route('admin.users.index')->with('error','Unable to create user.');
        }

        return redirect()->route('admin.users.index')->with('error','Unable to create user.');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = User::find($id);
        $roles = Role::where('guard_name', 'web')->pluck('name', 'id');
        $permissions = Permission::where('guard_name', 'web')->get('name', 'id');
        // echo '<pre>';print_r($permissions->toArray());exit;

        return view('admin.user.edit', compact('user', 'roles', 'permissions'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, User $user)
    {
        $this->validate($request, [
            'name' => 'bail|required|min:2',
            'email' => 'required|email|unique:admins,email,' . $user->id,
            'roles' => 'required|min:1'
        ]);

        // Update user
        $user->fill($request->except('roles', 'permissions', 'password'));

        // echo '<pre>';
        // print_r($admin->toArray());
        // exit;

        // check for password change
        if($request->get('password')) {
            $admin->password = bcrypt($request->get('password'));
        }

        // Handle the admin roles
        assignRoleAndPermissions($request, $user);

        $user->save();

        return redirect()->route('admin.users.index')->with('success','User has been updated.');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, $id)
    {
        $user = User::find($id);
        //if not found
        if(empty($user)){
            if ($request->ajax()) {
                return response()->json(['status' => 'error', 'msg' => "Some problem delete user."]);    
            }
            return Redirect::route('admin.users.index')->with('error','Some problem delete user.');
        }

        //delete
        $user->delete();
        if ($request->ajax()) {
            return response()->json(['status' => 'success', 'msg' => "User Deleted successfully."]);
        }
        return Redirect::route('admin.users.index')->with('success','User delete successfully.');
    }

    public function BulkRemove(Request $request)
    {
        $ids = $request->id;
        $users = User::whereIn('id',$ids)->get();
        if($users->isEmpty()){
            return response()->json(['status' => 'error', 'msg' => "Some problem delete user."]);
        }
        User::whereIn('id',$ids)->delete();
        return response()->json(['status' => 'success', 'msg' => "User deleted successfully."]);
    }

    public function Datatable(Request $request) {
        // $raw = Admin::query();
        $raw = User::with('roles')->get();
        // $raw = $users->reject(function ($user, $key) {
        //     return $user->hasRole('admin');
        // });
        $datatable = app('datatables')->of($raw)
        ->editColumn('created_at', function ($raw) {
            if($raw->created_at == null || $raw->created_at == ''){
                return '-';
            }else{
                // return date('Y/m/d', strtotime($raw->created_at->toFormattedDateString()));
                // return $raw->created_at->toFormattedDateString();
                return date('Y-m-d H:i:s A', strtotime($raw->created_at));
            }
        })
        ->addColumn('role', function ($raw) {
            // return removeDash($raw->roles->first()->name);
            return ArrayToString($raw->getRoleNames()->toArray());
        })
        ->addColumn('action', function ($raw) {
            $html = '';
            if(auth()->user()->can('edit_users')){
                $html .= '<a href="'.route("admin.users.edit", $raw->id).'" class="label theme-bg2 text-white f-12">Edit</a>';
            }
            if(auth()->user()->can('delete_users')){
                $html .= '<a href="javascript:void(0)" class="label theme-bg text-white f-12 delete-user-btn" data-url="'. route("admin.users.destroy", $raw->id) .'">Delete</a>';
            }
            return $html;
        });
        return $datatable->make(true);

    }
}
