<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Providers\RouteServiceProvider;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Illuminate\Http\Request;

use App\Models\ShoppingCart;

use Auth;
use Cart;
use Redirect;
use Helper;

class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = RouteServiceProvider::HOME;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest')->except('logout');
    }

    public function login(Request $request)
    {
        $this->validate($request, [
            'email' => 'required|email',
            'password' => 'required|string',
    	]);

        $credentials = $request->only('email', 'password');
        $authSuccess = Auth::attempt($credentials, $request->has('remember'));
        if($authSuccess) {
            //add cart item in user 'cart' table
            $cartContent = Cart::content();
            foreach($cartContent as $key => $cart){
                
                $shopping_cart = ShoppingCart::where('product_id', '=', $cart->options->product_data['id'])->first();
                if(empty($shopping_cart)){
                    $input = array(
                        'user_id' => Auth::user()->id,
                        'rowId' => $cart->rowId,
                        'product_id' => $cart->options->product_data['id'],
                        'product_name' => $cart->options->product_data['name'],
                        'product_image' => Helper::getFirstImageForCart($cart->options->product_data['product_images'], '80_80'),
                        'quantity' => $cart->qty,
                        'color' => $cart->options->color,
                        'size' => $cart->options->size,
                        'price' => $cart->price,
                        'subtotal' => $cart->price * $cart->qty,
                        'tax' => 0,
                        'cart_total' => $cart->price * $cart->qty,
                        'cart_data' => $cart,
                    );
                    ShoppingCart::create($input);
                }
            }
            // destroy cart data
            Cart::destroy();

            if ($request->ajax()) {
                $result = ['success' => true];
                return response($result);
            }else{
                return Redirect::back();
            }
        }else{
            if ($request->ajax()) {
                return
                    response([
                        'success' => false,
                        'message' => "Details do not match our records. Please re-enter."
                    ]);
            }else{
                return Redirect::back()->with('error', 'Invalid Credentials , Please try again.')->withInput($request->except('password'));
            }
        }
    }
}
