<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Providers\RouteServiceProvider;
use App\Models\User;
use Illuminate\Foundation\Auth\RegistersUsers;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;

use App\Models\ShoppingCart;

use Auth;
use Cart;
use Redirect;
use Helper;

class RegisterController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Register Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users as well as their
    | validation and creation. By default this controller uses a trait to
    | provide this functionality without requiring any additional code.
    |
    */

    use RegistersUsers;

    /**
     * Where to redirect users after registration.
     *
     * @var string
     */
    protected $redirectTo = RouteServiceProvider::HOME;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest');
    }

    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array  $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validator(array $data)
    {
        return Validator::make($data, [
            'name' => ['required', 'string', 'max:255'],
            'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
            'password' => ['required', 'string', 'min:8', 'confirmed'],
        ]);
    }

    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array  $data
     * @return \App\Models\User
     */
    protected function create(array $data)
    {
        $user = User::create([
            'first_name' => $data['name'],
            'email' => $data['email'],
            'password' => Hash::make($data['password']),
            'is_active' => true
        ]);

        //add cart item in user 'cart' table
        $cartContent = Cart::content();
        foreach($cartContent as $key => $cart){
            
            $shopping_cart = ShoppingCart::where('product_id', '=', $cart->options->product_data['id'])->first();
            if(empty($shopping_cart)){
                $input = array(
                    'user_id' => $user->id,
                    'rowId' => $cart->rowId,
                    'product_id' => $cart->options->product_data['id'],
                    'product_name' => $cart->options->product_data['name'],
                    'product_image' => Helper::getFirstImageForCart($cart->options->product_data['product_images'], '80_80'),
                    'quantity' => $cart->qty,
                    'color' => $cart->options->color,
                    'size' => $cart->options->size,
                    'price' => $cart->price,
                    'subtotal' => $cart->price * $cart->qty,
                    'tax' => 0,
                    'cart_total' => $cart->price * $cart->qty,
                    'cart_data' => $cart,
                );
                ShoppingCart::create($input);
            }
        }
        // destroy cart data
        Cart::destroy();

        return $user;

    }
}
