<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Admin\Product;
use App\Models\ShoppingCart;

use Auth;
use Cart;
use Redirect;
use Helper;

class CartController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return view('frontend.cart.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $quantity = $request->quantity;
        $color = $request->color;
        $size = $request->size;
        $product_id = $request->product_id;

        if($product_id == '' || $product_id == null){
            return response()->json(['status' => 'error', 'msg' => 'Product not found']);
        }

        $product_obj = Product::select('id', 'category_id', 'slug', 'category_id', 'name', 'price', 'old_price', 'quantity', 'product_attributes')->with('product_images')->find($product_id);
        // $product_obj = Product::with('product_images')->find($product_id);
        // echo '<pre>';print_r($product_obj);exit;
        if(empty($product_obj)){
            return response()->json(['status' => 'error', 'msg' => 'Product not found']);
        }

        if(Cart::count() == 10 ){
            return response()->json(['status' => 'checkout', 'msg' => 'Maximum quantity in the cart. You must have to checkout.']);
        }

        // if same product add
        foreach(Cart::content() as $cart){
            if($cart->id == $product_id && $cart->options->color == $cart && $cart->options->size == $size){
                $cart_id = $cart->id;
                $quantity = $cart->quantity + $quantity;
                Helper::cartUpdate($cart_id, $quantity);
            }
        }

        $price_array = Helper::getProductPriceFromAttribute($product_obj->product_attributes, $color, $size);

        $price = isset($price_array) && !empty($price_array) ? $price_array['price'] : 0;
        $single_array = [];
        $single_array['product_data'] = $product_obj->toArray();
        $single_array['product_category'] = $product_obj->category->name;
        $single_array['color'] = $color;
        $single_array['size'] = $size;

        $cart_array = [
            'id' => $product_obj->id,
            'name' => $product_obj->name,
            'qty' => $quantity,
            'price' => $price,
            'weight' => 0,
            'options' => $single_array
        ];

        // dd($cart_array);

        $html = '';
        $header_cart_footer = ''; 

        if(Auth::user()){
            //if User login
            $user_id = Auth::user()->id;   
            $cartContents = Cart::add($cart_array);
            $cart_subtotal = Cart::subtotal();
            $cart_total = Cart::total();

            $attribute_array = $cart_array['options'];            
            unset($attribute_array['product_data']);
            unset($attribute_array['product_category']);

            $match_array = ['product_id' => $cart_array['id']];

            $shopping_cart_query = ShoppingCart::where('product_id', '=', $cart_array['id']);
            
            foreach($attribute_array as $key => $attribute_value){
                $match_array[$key] = $attribute_value;
                $shopping_cart_query = $shopping_cart_query->where($key, '=', $attribute_value);
            }

            if($shopping_cart = $shopping_cart_query->first()){
                $new_quantity = $shopping_cart->quantity + $quantity;
                $subtotal = $shopping_cart->price * $new_quantity;
                $cart_total = $subtotal;
            }else{
                $new_quantity = $quantity;
                $subtotal = $price * $quantity;
                $cart_total = $subtotal;
            }

            $input = array(
                'user_id' => $user_id,
                'rowId' => $cartContents->rowId,
                'product_id' => $product_id,
                'product_name' => $cartContents->options->product_data['name'],
                'product_image' => Helper::getFirstImageForCart($cartContents->options->product_data['product_images'], '80_80'),
                'quantity' => $new_quantity,
                'color' => $cartContents->options->color,
                'size' => $cartContents->options->size,
                'price' => $price,
                'subtotal' => $subtotal,
                'tax' => 0,
                'cart_total' => $cart_total,
                'cart_data' => $cartContents,
            );

            ShoppingCart::updateOrCreate($match_array, $input);

            $shopping_cart = ShoppingCart::where('user_id', '=', $user_id)->get();

            foreach($shopping_cart as $key => $cart){
                $html .= '<li>';
                    $html .= '<a href="javascript:void(0)" class="item_remove remove_header_cart_item_btn" data-rowid="'.$cart->rowId.'"><i class="ion-close"></i></a>';
                    $html .= '<a href="'.route('get.product_by_slug', $cart->cart_data->options->product_data->slug).'">';
                    $html .= '<img src="'.asset($cart->product_image).'" alt="'.$cart->product_name.'"><p class="cart_product_title">'.$cart->product_name.'</p></a>';
                    $html .= '<p class="m-0"><span>('.$cart->color.') ('.$cart->size.')</span></p>';
                    $html .= '<span class="cart_quantity">'.$cart->quantity.' x <span class="cart_amount"> <span class="price_symbole">$</span></span>'.$cart->price.'</span>';
                $html .= ' </li>';
            }             

            $shopping_cart_count = 0;
            if(count($shopping_cart) > 0){
                $shopping_cart_count = ShoppingCart::where('user_id', '=', $user_id)->sum('quantity');
            }
            $subtotal = ShoppingCart::where('user_id', '=', $user_id)->sum('subtotal');

            $header_cart_footer .= '<p class="cart_total"><strong>Subtotal:</strong> <span class="cart_price"> <span class="price_symbole">$</span></span><label class="subtotal_label">'.$subtotal.'</label></p>';
            $header_cart_footer .='<p class="cart_buttons"><a href="'.route('carts.index').'" class="btn btn-fill-out rounded-0 view-cart">View Cart</a><a href="'.route('checkouts.index').'" class="btn btn-fill-out rounded-0 checkout">Checkout</a></p>';

            $data = [
                'cart_count' => $shopping_cart_count,
                'header_cart_footer' => $header_cart_footer,
                'html' => $html,
            ];

            return response()->json(['status' => 'success', 'msg' => ''.$product_obj->name.' is added to your cart.', 'data' => $data]);
            

        }else{
            //Guest
            $all_cart_items = Cart::content();
            
            $cartContents = Cart::add($cart_array);

            foreach(Cart::content() as $key => $cart){
                $html .= '<li>';
                    $html .= '<a href="javascript:void(0)" class="item_remove remove_header_cart_item_btn" data-rowid="'.$cart->rowId.'"><i class="ion-close"></i></a>';
                    $html .= '<a href="'.route('get.product_by_slug', $cart->options->product_data['slug']).'">';
                    $html .= '<img src="'.asset(Helper::getFirstImageForCart($cart->options->product_data['product_images'], '80_80')).'" alt="'.$cart->name.'"><p class="cart_product_title">'.$cart->name.'</p></a>';
                    $html .= '<p class="m-0"><span>('.$cart->options->color.') ('.$cart->options->size.')</span></p>';
                    $html .= '<span class="cart_quantity">'.$cart->qty.' x <span class="cart_amount"> <span class="price_symbole">$</span></span>'.Helper::setPriceFormat($cart->price).'</span>';
                $html .= ' </li>';
            }

            $header_cart_footer .= '<p class="cart_total"><strong>Subtotal:</strong> <span class="cart_price"> <span class="price_symbole">$</span></span><label class="subtotal_label">'.Cart::subtotal().'</label></p>';
            $header_cart_footer .='<p class="cart_buttons"><a href="'.route('carts.index').'" class="btn btn-fill-out rounded-0 view-cart">View Cart</a><a href="'.route('checkouts.index').'" class="btn btn-fill-out rounded-0 checkout">Checkout</a></p>';

            $data = [
                'cart_count' => Cart::count(),
                'header_cart_footer' => $header_cart_footer,
                'html' => $html,
            ];
            
            return response()->json(['status' => 'success', 'msg' => ''.$product_obj->name.' was added to your cart.', 'data' => $data]);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    public function cartUpdate(Request $request){
        // dd($request->all());
        $cart_id = $request->cart_id;
        $quantity = $request->quantity;

        $result = Helper::cartUpdate($cart_id, $quantity);
        return $result;
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, $rowId)
    {
        $header_cart_footer = '';

        if(Auth::user()){
            //if User login
            $user_id = Auth::user()->id;
            $shopping_cart = ShoppingCart::where('rowId', '=', $rowId)->first();
            if(!empty($shopping_cart)){
                $shopping_cart->delete();
                
                $shopping_cart_count = ShoppingCart::where('user_id', '=', $user_id)->count();
                $subtotal = ShoppingCart::where('user_id', '=', $user_id)->sum('subtotal');
                
                if($shopping_cart_count > 0){
                    $header_cart_footer .= '<p class="cart_total"><strong>Subtotal:</strong> <span class="cart_price"> <span class="price_symbole">$</span></span><label class="subtotal_label">'.$subtotal.'</abel></p>';
                    $header_cart_footer .='<p class="cart_buttons"><a href="'.route('carts.index').'" class="btn btn-fill-line rounded-0 view-cart">View Cart</a><a href="'.route('index').'" class="btn btn-fill-out rounded-0 checkout">Checkout</a></p>';
                }
                $data = [
                    'cart_count' => $shopping_cart_count,
                    'header_cart_footer' => $header_cart_footer,
                ];

                if ($request->ajax()) {
                    return response()->json(['status' => 'success', 'msg' => ''.$shopping_cart->product_name.' is removed to your cart.', 'data' => $data]);    
                }else{
                    return Redirect::route('carts.index')->with('success', ''.$shopping_cart->product_name.' is removed to your cart.');
                }
                
            }else{
                if ($request->ajax()) {
                    return response()->json(['status' => 'success', 'msg' => 'item not available in cart.']);
                }else{
                    return Redirect::route('carts.index')->with('success', 'item not available in cart.');
                }
            }

        }else{
            //Guest
            $cartContent = Cart::get($rowId);
            Cart::remove($rowId);

            if(!Cart::content()->isEmpty()){
                $header_cart_footer .= '<p class="cart_total"><strong>Subtotal:</strong> <span class="cart_price"> <span class="price_symbole">$</span></span><label class="subtotal_label">'.Cart::subtotal().'</abel></p>';
                $header_cart_footer .='<p class="cart_buttons"><a href="'.route('carts.index').'" class="btn btn-fill-line rounded-0 view-cart">View Cart</a><a href="'.route('index').'" class="btn btn-fill-out rounded-0 checkout">Checkout</a></p>';
            }
            $data = [
                'cart_count' => Cart::count(),
                'header_cart_footer' => $header_cart_footer,
            ];
            if ($request->ajax()) {
                return response()->json(['status' => 'success', 'msg' => ''.$cartContent->options->product_data['name'].' is removed from your cart.', 'data' => $data]);
            }else{
                return Redirect::route('carts.index')->with('success', ''.$cartContent->options->product_data['name'].' is removed from your cart.');
            }
        }
        
        // $header_cart_footer = '';
        // //Guest
        // $cartContent = Cart::get($rowId);
        // Cart::remove($rowId);

        // if(!Cart::content()->isEmpty()){
        //     $header_cart_footer .= '<p class="cart_total"><strong>Subtotal:</strong> <span class="cart_price"> <span class="price_symbole">$</span></span><label class="subtotal_label">'.Cart::subtotal().'</abel></p>';
        //     $header_cart_footer .='<p class="cart_buttons"><a href="'.route('carts.index').'" class="btn btn-fill-out rounded-0 view-cart">View Cart</a><a href="'.route('index').'" class="btn btn-fill-out rounded-0 checkout">Checkout</a></p>';
        // }
        // $data = [
        //     'cart_count' => Cart::count(),
        //     'header_cart_footer' => $header_cart_footer,
        // ];
        // if ($request->ajax()) {
        //     return response()->json(['status' => 'success', 'msg' => ''.$cartContent->options->product_data['name'].' is removed from your cart.', 'data' => $data]);
        // }else{
        //     return Redirect::route('carts.index')->with('success', ''.$cartContent->options->product_data['name'].' is removed from your cart.');
        // }
    }

    public function cartEmpty(){
        if(Auth::user()){
            //if User login
            $user_id = Auth::user()->id;
            $shopping_cart = ShoppingCart::where('user_id', '=', $user_id)->get();
            if(!$shopping_cart->isEmpty()){
                $shopping_cart = ShoppingCart::where('user_id', '=', $user_id)->delete();
                return Redirect::route('carts.index')->with('error', 'Your cart is empty.');
            }else{
                return Redirect::route('carts.index')->with('error', 'Your cart already empty.');
            }
        }else{
            //Guest
            $cartContent = Cart::content();
            if(!$cartContent->isEmpty()){
                Cart::destroy();
                return Redirect::route('carts.index')->with('error', 'Your cart is empty.');
            }else{
                return Redirect::route('carts.index')->with('error', 'Your cart already empty.');
            }
        }
    }
}
