<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use App\Models\Admin\Product;
use App\Models\Admin\Order;
use App\Models\Admin\OrderDetail;
use App\Models\Admin\Transaction;
use App\Models\ShoppingCart;

use Auth;
use Session;
use Cart;
use Mail;
use Redirect;
use Crypt;
use DB;
use Helper;

class CheckoutController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $product_ids = [];
        foreach(Cart::content() as $cart){
            $product_ids[] = $cart->id;
        }

        if(Helper::checkCart()){
            return Redirect::route('carts.index')->with('success', 'Your cart is empty.');
        }

        $user = Auth::user();
        return view('frontend.checkout.index', compact('product_ids', 'user'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $shopping_cart = $request->shopping_cart;
        $form_data = $request->form_data;
        $is_auth = filter_var($request->is_auth, FILTER_VALIDATE_BOOLEAN);
        // echo '<pre>';print_r($shopping_cart);
        // $subtotal = array_sum(array_map(function ($ar) {return $ar['qty'] * $ar['price'];}, $shopping_cart));
        // print_r($subtotal);
        // exit;
        try{
            if($is_auth){
                $quantity = array_sum(array_column($shopping_cart, 'quantity'));
                $subtotal = array_sum(array_map(function ($ar) {return $ar['quantity'] * $ar['price'];}, $shopping_cart));
                $tax = array_sum(array_column($shopping_cart, 'tax'));
                $grand_total = ($tax + array_sum(array_column($shopping_cart, 'subtotal')));
            }else{
                $quantity = array_sum(array_column($shopping_cart, 'qty'));
                $subtotal = array_sum(array_map(function ($ar) {return $ar['qty'] * $ar['price'];}, $shopping_cart));
                $discount = array_sum(array_column($shopping_cart, 'discount'));
                $tax = array_sum(array_column($shopping_cart, 'tax'));
                $grand_total = (($tax + array_sum(array_column($shopping_cart, 'subtotal'))) - $discount);
            }
            

            return DB::transaction(function() use($request, $shopping_cart, $quantity, $subtotal, $grand_total, $is_auth){
                $txHash = $request->txHash;
                $amount = $request->amount;
                $account_id = $request->account;

                $order = new Order();
                if($is_auth)
                    $order->user_id = Auth::id();
                else
                    $order->guest_id = mt_rand(100000, 999999);;
                $order->order_id = Helper::generateOrderNumber(1, 8);
                $order->quantity = $quantity;
                $order->subtotal = $subtotal;
                $order->grand_total = $grand_total;
                $order->payment_currency = Helper::getCurrency();
                $order->billing_address = $request->form_data;
                $order->shipping_address = $request->form_data;
                $order->payment_details = ['txHash' => $txHash, 'amount' => $amount, 'account_id' => $account_id];
                $order->cart_data = $shopping_cart;
                $order->status = 'success';
                $order->save();

                $order_id = $order->id;

                foreach($shopping_cart as $cart){
                    $order_details = new OrderDetail();
                    $order_details->order_id = $order_id;
                    $order_details->product_id = $is_auth ? $cart['product_id'] : $cart['id'];
                    $order_details->quantity = $is_auth ? $cart['quantity'] : $cart['qty'];
                    $order_details->price = $is_auth ? $cart['price'] : $cart['price'];
                    $order_details->grand_total = $order_details->quantity * $order_details->price;
                    $order_details->product_details = $is_auth ? Product::find($cart['product_id']) : $cart['options']['product_data'];
                    $order_details->cart_data = $is_auth ? $cart['cart_data'] : $cart;
                    $order_details->save();
                }

                $transection = new Transaction();
                $transection->order_id = $order_id;
                $transection->account_id = $account_id;
                $transection->amount = $amount;
                $transection->txhash = $txHash;
                $transection->payment_currency = Helper::getCurrency();
                $transection->save();

                session()->put('order_id', $order_id);
                if($is_auth)
                    ShoppingCart::where('user_id', '=', Auth::id())->delete();
                else
                    Cart::destroy();

                return response()->json(['status' => true, 'msg' => "Order placed successfully."]);
            });

            return response()->json(['status' => false, 'msg' => "Some problem for place order."]);

        } catch(\Illuminate\Database\QueryException $e){
            return response()->json(['status' => false, 'msg' => $e->getMessage()]);
        } catch (Exception $e) {
            return response()->json(['status' => false, 'msg' => $e->getMessage()]);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    public function success()
    {
        $order_id = session()->get('order_id');
        $order = Order::findOrfail($order_id);
        $orderDetails = OrderDetail::where('order_id', $order->id)->get();
        $transaction = Transaction::where('order_id', $order->id)->first();
        session()->forget('order_id');
        return view('frontend.checkout.success', compact('orderDetails', 'order', 'transaction'));
    }
}
