<?php

namespace App\Models\Admin;

use App\Notifications\AdminResetPassword;
use Illuminate\Notifications\Notifiable;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Spatie\Permission\Traits\HasRoles;

use Cviebrock\EloquentSluggable\Sluggable;
use Illuminate\Support\Facades\Crypt;

use Illuminate\Database\Eloquent\Casts\Attribute;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;

class Admin extends Authenticatable
{
    use Notifiable, HasRoles;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name', 'email', 'password', 'qrcode_base64', 'google2fa_secret', 'is_mfa', 'is_active'
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];

    public function sluggable(): array
    {
        return [
            'slug' => [
                'source' => 'firstname'
            ]
        ];
    }

    protected function google2faSecret(): Attribute
    {
        return Attribute::make(
            get: fn ($value) => ($value != '') ? Crypt::decryptString($value) : $value,
            set: fn ($value) => ($value != '') ? Crypt::encryptString($value) : $value,
        );
    }


    public function scopeActive($query)
    {
        return $query->where('is_active', '=', true);
    }

    public function scopeTwoFAEnable($query)
    {
        return $query->where('is_mfa', '=', true);
    }

    /**
     * Send the password reset notification.
     *
     * @param  string  $token
     * @return void
     */
    public function sendPasswordResetNotification($token)
    {
        $this->notify(new AdminResetPassword($token));
    }
}
