<?php

namespace App\Models\Admin;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Cviebrock\EloquentSluggable\Sluggable;

use Illuminate\Database\Eloquent\Casts\Attribute as LAttribute;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;

class Attribute extends Model
{
    use HasFactory;

    protected $fillable = [
        'category_id', 'code', 'name', 'frontend_type', 'is_filterable', 'is_required', 'is_active'
    ];

    public function scopeActive($query)
    {
        return $query->where('is_active', '=', '1');
    }

    public function scopeDeactive($query)
    {
        return $query->where('is_active', '=', '0');
    }

    protected function name(): LAttribute
    {
        return LAttribute::make(
            set: fn ($value) => strtolower($value),
            get: fn ($value) => ucwords($value),
        );
    }

    public function category() :BelongsTo
    {
        return $this->belongsTo(Category::class);
    }

    public function attribute_values() :HasMany
    {
        return $this->hasMany(AttributeValue::class);
    }
}
