<?php

namespace App\Models\Admin;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Cviebrock\EloquentSluggable\Sluggable;

use Illuminate\Database\Eloquent\Casts\Attribute;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;

class Blog extends Model
{
    use HasFactory, Sluggable;

    protected $fillable = [
        'blog_category_id', 'slug', 'title', 'description', 'image', 'meta_title', 'meta_description', 'meta_keywords', 'meta_author', 'is_active'
    ];

    public function sluggable(): array
    {
        return [
            'slug' => [
                'source' => 'title'
            ]
        ];
    }

    public function scopeActive($query)
    {
        return $query->where('is_active', '=', true);
    }

    public function scopeDeactive($query)
    {
        return $query->where('is_active', '=', false);
    }

    public function blog_category(): BelongsTo
    {
        return $this->belongsTo(BlogCategory::class);
    }
}
