<?php

namespace App\Models\Admin;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Cviebrock\EloquentSluggable\Sluggable;

use Illuminate\Database\Eloquent\Casts\Attribute;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;

class Category extends Model
{
    use HasFactory, Sluggable;

    protected $fillable = ['parent_id', 'slug', 'name', 'color_code', 'meta_title', 'meta_author', 'meta_keyword', 'meta_description', 'status'];

    public function sluggable(): array
    {
        return [
            'slug' => [
                'source' => 'name'
            ]
        ];
    }

    protected function name(): Attribute
    {
        return Attribute::make(
            get: fn ($value) => ucfirst($value),
            set: fn ($value) => strtolower($value),
        );
    }

    public function scopeActive($query)
    {
        return $query->where('status', '=', '1');
    }

    public function scopeDeactive($query)
    {
        return $query->where('status', '=', '0');
    }

    public function child()
    {
    	return $this->hasMany(Category::class, 'parent_id');
    }

    public function parent()
    {
    	return $this->belongsTo(Category::class, 'parent_id');
    }

    public function subcategory()
    {
    	return $this->child()->where('status', '=', '1');
    }

    public function products()
    {
        return $this->hasMany(Product::class);
    }
}
