<?php

namespace Database\Seeders;

// use Illuminate\Database\Console\Seeds\WithoutModelEvents;
use Illuminate\Database\Seeder;

use App\Models\Admin\Admin;
use App\Models\Admin\Role;
use App\Models\Admin\Permission;

use Laravel\Passport\ClientRepository;

use BaconQrCode\Renderer\ImageRenderer;
use BaconQrCode\Renderer\Image\ImagickImageBackEnd;
use BaconQrCode\Renderer\RendererStyle\RendererStyle;
use BaconQrCode\Writer;

use Helper;
use Str;

class DatabaseSeeder extends Seeder
{
    /**
     * Seed the application's database.
     */
    public function run(): void
    {
        // \App\Models\User::factory(10)->create();

        // \App\Models\User::factory()->create([
        //     'name' => 'Test User',
        //     'email' => 'test@example.com',
        // ]);

        if ($this->command->confirm('Do you wish to refresh migration before seeding, it will clear all old data ?')) {
            $this->command->call('migrate:refresh');
            $this->command->warn("Data cleared, starting from blank database.");

            $this->call(RoleSeeder::class);
            $this->call(PermissionSeeder::class);
            $this->call(AdminsTableSeeder::class);
            $this->call(ModelHasRolesTableSeeder::class);
            $this->call(CategoriesTableSeeder::class);
            $this->call(AttributesTableSeeder::class);
            $this->call(AttributeValuesTableSeeder::class);
            $this->call(SettingsTableSeeder::class);
            $this->call(ProductsTableSeeder::class);
            $this->call(ProductImagesTableSeeder::class);
            $this->call(FootersTableSeeder::class);
            $this->call(LandingPagesTableSeeder::class);
            $this->call(PagesTableSeeder::class);

            $roles = Role::where('guard_name' , '=', 'admin')->get();
            foreach($roles as $role){
                $this->command->info($role->name);
                if( $role->name == 'admin' ) {
                    $role->syncPermissions(Permission::where('guard_name' , '=', 'admin')->get());
                    $this->command->info('Admin granted all the permissions');
                }
                // $this->createAdmin($role);
            }
        }
    }

    private function createAdmin($role)
    {
        if( $role->name == 'admin') {
            $user = admin::create([
                'name' => 'admin',
                'email' => 'admin@gmail.com',
                'password' => bcrypt('123456'),
                'remember_token' => str_random(10),
            ]);
            // Helper::saveUserQrcodeBase64($user);
            $user->assignRole($role->name);
            $this->command->info('Here is your admin details to login:');
            $this->command->warn('admin@gmail.com');
            $this->command->warn('Password is "123456"');

        }
        else if ($role->name == 'sub_admin') {
            $user = admin::create([
                'name' => 'sub_admin',
                'email' => 'sub_admin@gmail.com',
                'password' => bcrypt('123456'),
                'remember_token' => str_random(10),
            ]);
            // Helper::saveUserQrcodeBase64($user);
            $user->assignRole($role->name);
            $this->command->info('Here is your sub_admin details to login:');
            $this->command->warn('sub_admin@gmail.com');
            $this->command->warn('Password is "123456"');

        }
        else if ($role->name == 'seo') {
            $user = admin::create([
                'name' => 'seo',
                'email' => 'seo@gmail.com',
                'password' => bcrypt('123456'),
                'remember_token' => str_random(10),
            ]);
            // Helper::saveUserQrcodeBase64($user);
            $user->assignRole($role->name);
            $this->command->info('Here is your seo details to login:');
            $this->command->warn('seo@gmail.com');
            $this->command->warn('Password is "123456"');

        }

        $google2fa = app('pragmarx.google2fa');
        $prefix = Str::padRight($user->id, 10, 'X');
        $secret = $google2fa->generateSecretKey(16, $prefix);
        $user->google2fa_secret = $secret;

        $g2faUrl = $google2fa->getQRCodeUrl(config('app.name'), $user->email, $secret);
        $writer = new Writer(
            new ImageRenderer(
                new RendererStyle(400),
                new ImagickImageBackEnd()
            )
        );
        $qr_code = base64_encode($writer->writeString($g2faUrl));

        $user->qrcode_base64 = $qr_code;

        $user->save();


    }

}
